#! /usr/bin/env python
"""Auxilary functions for plots

    Module holds functions that are used for several plots.


Authors
-------
    - Daniel Kühbacher

Use
---


Dependencies
------------

"""
import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.sql_interface as sql
from bokeh.plotting import figure
from bokeh.models import BoxAnnotation, LinearAxis, Range1d
from bokeh.embed import components
from bokeh.models.widgets import Panel, Tabs
from bokeh.models import ColumnDataSource, HoverTool, DatetimeTickFormatter, DatetimeTicker, SingleIntervalTicker
from bokeh.models.formatters import TickFormatter
from bokeh.models.tools import PanTool, SaveTool

import pandas as pd
import numpy as np

from astropy.time import Time


def pol_regression(x, y, rank):
    ''' Calculate polynominal regression of certain rank
    Parameters
    ----------
    x : list
        x parameters for regression
    y : list
        y parameters for regression
    rank : int
        rank of regression
    Return
    ------
    y_poly : list
        regression y parameters
    '''
    z = np.polyfit(x, y, rank)
    f = np.poly1d(z)
    y_poly = f(x)
    return y_poly

def add_hover_tool(p, rend):
    ''' Append hover tool to plot
    parameters
    ----------
    p : bokeh figure
        declares where to append hover tool
    rend : list
        list of renderer to append hover tool
    '''

    from bokeh.models import HoverTool

    #activate HoverTool for scatter plot
    hover_tool = HoverTool( tooltips =
    [
        ('Name', '$name'),
        ('Count', '@data_points'),
        ('Mean', '@average'),
        ('Deviation', '@deviation'),
    ], renderers = rend)
    #append hover tool
    p.tools.append(hover_tool)

def add_limit_box(p, lower, upper, alpha = 0.1, color="green"):
    ''' Adds box to plot
    Parameters
    ----------
    p : bokeh figure
        declares where to append hover tool
    lower : float
        lower limit of box
    upper : float
        upper limit of box
    alpha : float
        transperency of box
    color : str
        filling color
    '''
    box = BoxAnnotation(bottom = lower, top = upper, fill_alpha = alpha, fill_color = color)
    p.add_layout(box)

def add_to_plot(p, legend, mnemonic, start, end, conn, y_axis= "default", color="red", err='y'):
    '''Add scatter and line to certain plot and activates hoover tool
    Parameters
    ----------
    p : bokeh object
        defines plot where line and scatter should be added
    legend : str
        will be showed in legend of plot
    mnemonic : str
        defines mnemonic to be plotted
    start : datetime
        sets start time for data query
    end : datetime
        sets end time for data query
    conn : DBobject
        connection object to database
    y_axis : str (default='default')
        used if secon y axis is provided
    color : str (default='dred')
        defines color for scatter and line plot
    Return
    ------
    scat : plot scatter object
        used for applying hovertools o plots
    '''

    #convert given start and end time to astropy time
    start_str = str(Time(start).mjd)
    end_str = str(Time(end).mjd)

    #prepare and execute sql query
    sql_c = "SELECT * FROM "+mnemonic+" WHERE start_time BETWEEN "+start_str+" AND "+end_str+" ORDER BY start_time"
    temp = pd.read_sql_query(sql_c, conn)

    #put data into Dataframe and define ColumnDataSource for each plot
    #reg = pd.DataFrame({'reg' : pol_regression(temp['start_time'], temp['average'],3)})
    #temp = pd.concat([temp, reg], axis = 1)
    temp['start_time'] = pd.to_datetime( Time(temp['start_time'], format = "mjd").datetime )
    plot_data = ColumnDataSource(temp)

    #plot data
    p.line(x = "start_time", y = "average", color = color, y_range_name=y_axis, legend = legend, source = plot_data)
    scat = p.scatter(x = "start_time", y = "average", name = mnemonic, color = color, y_range_name=y_axis, legend = legend, source = plot_data)

    #generate error lines if wished
    if err != 'n':
        #generate error bars
        err_xs = []
        err_ys = []

        for index, item in temp.iterrows():
            err_xs.append((item['start_time'], item['start_time']))
            err_ys.append((item['average'] - item['deviation'], item['average'] + item['deviation']))

        # plot them
        p.multi_line(err_xs, err_ys, color = color, legend = legend)

    return scat

def add_to_plot_normalized(p, legend, mnemonic, start, end, conn, nominal, color = "red"):
    '''Add line plot to figure (for wheelpositions)
    Parameters
    ----------
    p : bokeh object
        defines figure where line schould be plotted
    legend : str
        will be showed in legend of plot
    mnemonic : str
        defines mnemonic to be plotted
    start : datetime
        sets start time for data query
    end : datetime
        sets end time for data query
    conn : DBobject
        connection object to database
    color : str (default='dred')
        defines color for scatter and line plot
    '''

    start_str = str(Time(start).mjd)
    end_str = str(Time(end).mjd)

    sql_c = "SELECT * FROM "+mnemonic+" WHERE timestamp BETWEEN "+start_str+" AND "+end_str+" ORDER BY timestamp"
    temp = pd.read_sql_query(sql_c, conn)

    #normalize values
    temp['value'] -= nominal
    #temp['value'] -= 1

    temp['timestamp'] = pd.to_datetime( Time(temp['timestamp'], format = "mjd").datetime )
    plot_data = ColumnDataSource(temp)

    p.line(x = "timestamp", y = "value", color = color, legend = legend, source = plot_data)
    p.scatter(x = "timestamp", y = "value", color = color, legend = legend, source = plot_data)

def add_basic_layout(p):
    '''Add basic layout to certain plot
    Parameters
    ----------
    p : bokeh object
        defines plot where line and scatter should be added
    '''
    p.title.align = "left"
    p.title.text_color = "#c85108"
    p.title.text_font_size = "25px"
    p.background_fill_color = "#efefef"

    p.xaxis.axis_label_text_font_size = "14pt"
    p.xaxis.axis_label_text_color ='#2D353C'
    p.yaxis.axis_label_text_font_size = "14pt"
    p.yaxis.axis_label_text_color = '#2D353C'

    p.xaxis.major_tick_line_color = "firebrick"
    p.xaxis.major_tick_line_width = 2
    p.xaxis.minor_tick_line_color = "#c85108"



"""Django settings for ``jwql`` project.

Contains essential project settings, including a list of installed
apps, where to find templates, credentials for connection to the
``db.sqlite3`` database, time zone, & locations where static files are
located. Generated by ``django-admin startproject`` using Django 2.0.1.

Authors
-------

    - Lauren Chambers

References
----------

    For more information on this file, see
        ``https://docs.djangoproject.com/en/2.0/topics/settings/``
    For the full list of settings and their values, see
        ``https://docs.djangoproject.com/en/2.0/ref/settings/``

Dependencies
------------

    The user must have a configuration file named ``config.json``
    placed in the ``jwql/utils/`` directory.
"""

import os

from jwql.utils.utils import get_config


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'nx4xai#69@7pfb@l182z9aa#h8dfoms0$eitcrt5!77en*8(y4'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True


ALLOWED_HOSTS = ['*']

# Application definition
INSTALLED_APPS = [
    'jwql.website.apps.jwql',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'jwql.website.jwql_proj.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.jinja2.Jinja2',
        'DIRS': [os.path.join(BASE_DIR, 'apps', 'jwql', 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'environment': 'jwql.website.jwql_proj.jinja2.environment',
            'extensions': ['jwql.website.jwql_proj.jinja2.DjangoNow'],
            'context_processors': ['jwql.website.apps.jwql.context_processors.base_context'],
        },
    }
]

WSGI_APPLICATION = 'jwql.website.jwql_proj.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.0/ref/settings/#databases
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    },
}

# Password validation
# https://docs.djangoproject.com/en/2.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'EST'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.0/howto/static-files/

STATIC_URL = '/static/'

STATICFILES_DIRS = [
    os.path.join(BASE_DIR, "static/"),
    get_config()['jwql_dir']
]



#! /usr/bin/env python
"""Combines plots to tabs and prepares dashboard

The module imports all prepares plot functions from .plots and combines
prebuilt tabs to a dashboard. Furthermore it defines the timerange for
the visualisation. Default time_range should be set to about 4 Month (120days)

Authors
-------
    - Daniel Kühbacher

Use
---
    The functions within this module are intended to be imported and
    used by ``data_container.py``, e.g.:

    ::
        import jwql.instrument_monitors.miri_monitors.data_trending.dashboard as dash
        dashboard, variables = dash.data_trending_dashboard(start_time, end_time)

Dependencies
------------
    User must provide "nirspec_database.db" in folder jwql/database

"""
import os
import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.sql_interface as sql
from jwql.utils.utils import get_config, filename_parser

from bokeh.embed import components
from bokeh.models.widgets import Tabs
from bokeh.resources import Resources
from bokeh.io.state import curstate

from astropy.time import Time
import datetime
from datetime import date

#import plot functions
from .plots.power_tab import power_plots
from .plots.voltage_tab import volt_plots
from .plots.temperature_tab import temperature_plots
from .plots.msa_mce_tab import msa_mce_plots
from .plots.fpe_fpa_tab import fpe_fpa_plots
from .plots.caa_tab import caa_plots
from .plots.wheel_tab import wheel_pos

#configure actual datetime in order to implement range function
now = datetime.datetime.now()
#default_start = now - datetime.timedelta(1000)
default_start = datetime.date(2017, 8, 15).isoformat()

__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))

def data_trending_dashboard(start = default_start, end = now):
    """Bulilds dashboard
    Parameters
    ----------
    start : time
        configures start time for query and visualisation
    end : time
        configures end time for query and visualisation
    Return
    ------
    plot_data : list
        A list containing the JavaScript and HTML content for the dashboard
    variables : dict
        no use
    """

    #connect to database
    DATABASE_LOCATION = os.path.join(get_config()['jwql_dir'], 'database')
    DATABASE_FILE = os.path.join(DATABASE_LOCATION, 'nirspec_database.db')

    conn = sql.create_connection(DATABASE_FILE)

    #some variables can be passed to the template via following
    variables = dict(init = 1)

    #some variables can be passed to the template via following
    variables = dict(init = 1)

    #add tabs to dashboard
    tab1 = power_plots(conn, start, end)
    tab2 = volt_plots(conn, start, end)
    tab3 = temperature_plots(conn, start, end)
    tab4 = msa_mce_plots(conn, start, end)
    tab5 = fpe_fpa_plots(conn, start, end)
    tab6 = caa_plots(conn, start, end)
    tab7 = wheel_pos(conn, start, end)

    #build dashboard
    tabs = Tabs( tabs=[ tab1, tab2, tab3, tab4, tab5, tab6, tab7] )
    #tabs = Tabs( tabs=[ tab1, tab7] )

    #return dasboard to webapp
    script, div = components(tabs)
    plot_data = [div, script]

    #close sql connection
    sql.close_connection(conn)

    return plot_data, variables



#! /usr/bin/env python
"""Module for importing and sorting mnemonics

This module imports a whole set of mnemonics from a .CSV sheet and converts it
to an astropy table. In a second step the table is sorted by its mnemoncis
and for each mnemmonic another astropy table with reduced content is created.
The last step is to append the data (time and engineering value) with its
mnemonic identifier as key to a dictionary.

Authors
-------
    - Daniel Kühbacher

Use
---


Dependencies
------------
    mnemonics.py -> includes a list of mnemonics to be evaluated

References
----------

Notes
-----

"""
from astropy.table import Table
from astropy.time import Time
import warnings
import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.mnemonics as mn


class mnemonics:
    """class to hold a set of mnemonics"""

    __mnemonic_dict = {}

    def __init__(self, import_path):
        """main function of this class
        Parameters
        ----------
        import_path : str
            defines file to import (csv sheet)
        """
        imported_data = self.import_CSV(import_path)
        length = len(imported_data)

        print('{} was imported - {} lines'.format(import_path, length))

        #look for every mnmonic given in mnemonicy.py
        for mnemonic_name in mn.mnemonic_set_query:
            temp = self.sort_mnemonic(mnemonic_name, imported_data)
            #append temp to dict with related mnemonic
            if temp != None:
                self.__mnemonic_dict.update({mnemonic_name:temp})
            else:
                warnings.warn("fatal error")


    def import_CSV(self, path):
        """imports csv sheet and converts it to AstropyTable
        Parameters
        ----------
        path : str
            defines path to file to import
        Return
        ------
        imported_data : AstropyTable
            container for imported data
        """
        #read data from given *CSV file
        imported_data=Table.read(path, format='ascii.basic', delimiter=',')
        return imported_data


    #returns table of single mnemonic
    def mnemonic(self, name):
        """Returns table of one single mnemonic
        Parameters
        ----------
        name : str
            name of mnemonic
        Return
        ------
        __mnemonic_dict[name] : AstropyTable
            corresponding table to mnemonic name
        """
        try:
            return self.__mnemonic_dict[name]
        except KeyError:
            print('{} not in list'.format(name))


    #looks for given mnemonic in given table
    #returns list containing astropy tables with sorted mnemonics and engineering values
    #adds useful meta data to Table
    def sort_mnemonic(self, mnemonic, table):
        """Looks for all values in table with identifier "mnemonic"
           Converts time string to mjd format
        Parameters
        ----------
        mnemonic : str
            identifies which mnemonic to look for
        table : AstropyTable
            table that stores mnemonics and data
        Return
        ------
        mnemonic_table : AstropyTable
            stores all data associated with identifier "mnemonic"
        """

        temp1: float = []
        temp2 = []

        #appends present mnemonic data to temp arrays temp1 and temp2
        for item in table:
            try:
                if item['Telemetry Mnemonic'] == mnemonic:
                    #convert time string to mjd format
                    temp = item['Secondary Time'].replace('/','-').replace(' ','T')
                    t = Time(temp, format='isot')

                    temp1.append(t.mjd)
                    temp2.append(item['EU Value'])
            except KeyError:
                warnings.warn("{} is not in mnemonic table".format(mnemonic))

        description = ('time','value')
        data = [temp1, temp2]

        #add some meta data
        if len(temp1) > 0:
            date_start = temp1[0]
            date_end = temp1[len(temp1)-1]
            info = {'start':date_start, 'end':date_end}
        else:
            info = {"n":"n"}

        #add name of mnemonic to meta data of list
        info['mnemonic'] = mnemonic
        info['len'] = len(temp1)

        #table to return
        mnemonic_table = Table(data, names = description, \
                        dtype = ('f8','str'), meta = info)
        return mnemonic_table

if __name__ =='__main__':
    pass



"""Globally defined and used variables for the ``jwql`` project.

Authors
-------

    - Johannes Sahlmann

Use
---
    This variables within this module are intended to be directly
    imported, e.g.:
    ::

        from jwql.utils.constants import JWST_INSTRUMENT_NAMES

References
----------

    Many variables were transferred from an earlier version of
    ``utils.py``
"""

# Defines the x and y coordinates of amplifier boundaries
AMPLIFIER_BOUNDARIES = {'nircam': {'1': [(0, 0), (512, 2048)], '2': [(512, 0), (1024, 2048)],
                                   '3': [(1024, 0), (1536, 2048)], '4': [(1536, 0), (2048, 2048)]}
                        }

# Possible suffix types for nominal files
GENERIC_SUFFIX_TYPES = ['uncal', 'cal', 'rateints', 'rate', 'trapsfilled', 'i2d',
                        'x1dints', 'x1d', 's2d', 's3d', 'dark', 'crfints',
                        'crf', 'ramp', 'fitopt', 'bsubints', 'bsub', 'cat']

# Possible suffix types for guider exposures
GUIDER_SUFFIX_TYPES = ['stream', 'stacked_uncal', 'image_uncal', 'stacked_cal', 'image_cal']

# JWST data products
JWST_DATAPRODUCTS = ['IMAGE', 'SPECTRUM', 'SED', 'TIMESERIES', 'VISIBILITY',
                     'EVENTLIST', 'CUBE', 'CATALOG', 'ENGINEERING', 'NULL']

# Lowercase JWST instrument names
JWST_INSTRUMENT_NAMES = sorted(['niriss', 'nircam', 'nirspec', 'miri', 'fgs'])

# JWST instrument names with shorthand notation
JWST_INSTRUMENT_NAMES_SHORTHAND = {'gui': 'fgs',
                                   'mir': 'miri',
                                   'nis': 'niriss',
                                   'nrc': 'nircam',
                                   'nrs': 'nirspec'}

# Mixed case JWST instrument names
JWST_INSTRUMENT_NAMES_MIXEDCASE = {'fgs': 'FGS',
                                   'miri': 'MIRI',
                                   'nircam': 'NIRCam',
                                   'niriss': 'NIRISS',
                                   'nirspec': 'NIRSpec'}

# Upper case JWST instrument names
JWST_INSTRUMENT_NAMES_UPPERCASE = {key: value.upper() for key, value in
                                   JWST_INSTRUMENT_NAMES_MIXEDCASE.items()}

# Astoquery service string for each JWST instrument
JWST_MAST_SERVICES = ['Mast.Jwst.Filtered.{}'.format(value.title()) for value in
                      JWST_INSTRUMENT_NAMES]

# Available monitor names and their location for each JWST instrument
MONITORS = {
    'fgs': [('Bad Pixel Monitor', '#')],
    'miri': [('Dark Current Monitor', '#'),
             ('Data Trending', '/miri/miri_data_trending'),
             ('Bad Pixel Monitor', '#'),
             ('Cosmic Ray Monitor', '#'),
             ('Photometry Monitor', '#'),
             ('TA Failure Monitor', '#'),
             ('Blind Pointing Accuracy Monitor', '#'),
             ('Filter and Calibration Lamp Monitor', '#'),
             ('Thermal Emission Monitor', '#')],
    'nircam': [('Bias Monitor', '#'),
               ('Readnoise Monitor', '#'),
               ('Gain Level Monitor', '#'),
               ('Mean Dark Current Rate Monitor', '#'),
               ('Photometric Stability Monitor', '#')],
    'niriss': [('Bad Pixel Monitor', '#'),
               ('Readnoise Monitor', '#'),
               ('AMI Calibrator Monitor', '#'),
               ('TSO RMS Monitor', '#')],
    'nirspec': [('Optical Short Monitor', '#'),
                ('Target Acquisition Monitor', '#'),
                ('Data Trending', '/nirspec/nirspec_data_trending'),
                ('Detector Health Monitor', '#'),
                ('Ref Pix Monitor', '#'),
                ('Internal Lamp Monitor', '#'),
                ('Instrument Model Updates', '#'),
                ('Failed-open Shutter Monitor', '#')]}

# Possible suffix types for coronograph exposures
NIRCAM_CORONAGRAPHY_SUFFIX_TYPES = ['psfstack', 'psfalign', 'psfsub']

# NIRCam subarrays that use four amps for readout
NIRCAM_FOUR_AMP_SUBARRAYS = ['WFSS128R', 'WFSS64R']

# NIRCam long wavelength detector names
NIRCAM_LONGWAVE_DETECTORS = ['NRCA5', 'NRCB5']

# NIRCam short wavelength detector names
NIRCAM_SHORTWAVE_DETECTORS = ['NRCA1', 'NRCA2', 'NRCA3', 'NRCA4',
                              'NRCB1', 'NRCB2', 'NRCB3', 'NRCB4']

# NIRCam subarrays that use either one or four amps
NIRCAM_SUBARRAYS_ONE_OR_FOUR_AMPS = ['SUBGRISMSTRIPE64', 'SUBGRISMSTRIPE128', 'SUBGRISMSTRIPE256']

# Possible suffix types for AMI files
NIRISS_AMI_SUFFIX_TYPES = ['amiavg', 'aminorm', 'ami']

# Possible suffix types for time-series exposures
TIME_SERIES_SUFFIX_TYPES = ['phot', 'whtlt']

# Concatenate all suffix types (ordered to ensure successful matching)
FILE_SUFFIX_TYPES = GUIDER_SUFFIX_TYPES + GENERIC_SUFFIX_TYPES + \
                    TIME_SERIES_SUFFIX_TYPES + NIRCAM_CORONAGRAPHY_SUFFIX_TYPES + \
                    NIRISS_AMI_SUFFIX_TYPES



#! /usr/bin/env python
"""Prepares plots for WHEEL tab

    Module prepares plots for mnemonics below. Combines plots in a grid and
    returns tab object.

    Plot 1 - Filterwheel
    INRSI_FWA_MECH_POS
    INRSI_C_FWA_POSITION

    Plot 2 - Gratingwheel X
    INRSI_GWA_MECH_POS
    INRSI_C_GWA_X_POSITION

    Plot 3 - Gratingwheel Y
    INRSI_GWA_MECH_POS
    INRSI_C_GWA_Y_POSITION

Authors
-------
    - Daniel Kühbacher

Use
---
    The functions within this module are intended to be imported and
    used by ``dashboard.py``, e.g.:

    ::
        from .plots.wheel_ratio_tab import wheel_plots
        tab = wheel_plots(conn, start, end)

Dependencies
------------
    User must provide database "miri_database.db"

"""

import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.sql_interface as sql
import jwql.instrument_monitors.nirspec_monitors.data_trending.plots.plot_functions as pf
import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.mnemonics as mn
from bokeh.plotting import figure
from bokeh.models import BoxAnnotation, LinearAxis, Range1d
from bokeh.embed import components
from bokeh.models.widgets import Panel, Tabs, Div
from bokeh.models import ColumnDataSource
from bokeh.layouts import column, row, WidgetBox

import pandas as pd
import numpy as np

from astropy.time import Time


def fw(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",       \
                toolbar_location = "above",                         \
                plot_width = 1120,                                   \
                plot_height = 500,                                  \
                y_range = [-3,3],                                   \
                x_axis_type = 'datetime',                           \
                x_axis_label = 'Date', y_axis_label = 'mV (normalized)')

    p.grid.visible = True
    p.title.text = "Filterwheel"
    p.title.align = "left"
    pf.add_basic_layout(p)

    pf.add_to_plot_normalized(p, "F110W", "INRSI_C_FWA_POSITION_F110W", start, end, conn, mn.fw_nominals['F110W'], color = "green")
    pf.add_to_plot_normalized(p, "F100LP", "INRSI_C_FWA_POSITION_F100LP", start, end, conn, mn.fw_nominals['F100LP'], color = "red")
    pf.add_to_plot_normalized(p, "F140X", "INRSI_C_FWA_POSITION_F140X", start, end, conn, mn.fw_nominals['F140X'], color = "blue")
    pf.add_to_plot_normalized(p, "OPAQUE", "INRSI_C_FWA_POSITION_OPAQUE", start, end, conn, mn.fw_nominals['OPAQUE'], color = "orange")
    pf.add_to_plot_normalized(p, "F290LP", "INRSI_C_FWA_POSITION_F290LP", start, end, conn, mn.fw_nominals['F290LP'], color = "purple")
    pf.add_to_plot_normalized(p, "F170LP", "INRSI_C_FWA_POSITION_F170LP", start, end, conn, mn.fw_nominals['F170LP'], color = "brown")
    pf.add_to_plot_normalized(p, "CLEAR", "INRSI_C_FWA_POSITION_CLEAR", start, end, conn, mn.fw_nominals['CLEAR'], color = "chocolate")
    pf.add_to_plot_normalized(p, "F070LP", "INRSI_C_FWA_POSITION_F070LP", start, end, conn, mn.fw_nominals['F070LP'], color = "darkmagenta")

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"
    p.legend.orientation = 'horizontal'
    return p


def gwx(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",       \
                toolbar_location = "above",                         \
                plot_width = 1120,                                   \
                plot_height = 500,                                  \
                y_range = [-4,4],                                   \
                x_axis_type = 'datetime',                           \
                x_axis_label = 'Date', y_axis_label = 'mV (normalized)')

    p.grid.visible = True
    p.title.text = "Gratingwheel X"
    p.title.align = "left"
    pf.add_basic_layout(p)

    pf.add_to_plot_normalized(p, "PRISM", "INRSI_C_GWA_X_POSITION_PRISM", start, end, conn, mn.gwx_nominals['PRISM'], color = "green")
    pf.add_to_plot_normalized(p, "MIRROR", "INRSI_C_GWA_X_POSITION_MIRROR", start, end, conn, mn.gwx_nominals['MIRROR'], color = "blue")
    pf.add_to_plot_normalized(p, "G140H", "INRSI_C_GWA_X_POSITION_G140H", start, end, conn, mn.gwx_nominals['G140H'], color = "red")
    pf.add_to_plot_normalized(p, "G235H", "INRSI_C_GWA_X_POSITION_G235H", start, end, conn, mn.gwx_nominals['G235H'], color = "purple")
    pf.add_to_plot_normalized(p, "G395H", "INRSI_C_GWA_X_POSITION_G395H", start, end, conn, mn.gwx_nominals['G395H'], color = "orange")
    pf.add_to_plot_normalized(p, "G140M", "INRSI_C_GWA_X_POSITION_G140M", start, end, conn, mn.gwx_nominals['G140M'], color = "brown")
    pf.add_to_plot_normalized(p, "G235M", "INRSI_C_GWA_X_POSITION_G235M", start, end, conn, mn.gwx_nominals['G235M'], color = "darkmagenta")
    pf.add_to_plot_normalized(p, "G395M", "INRSI_C_GWA_X_POSITION_G395M", start, end, conn, mn.gwx_nominals['G395M'], color = "darkcyan")

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"
    p.legend.orientation = 'horizontal'

    return p

def gwy(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",       \
                toolbar_location = "above",                         \
                plot_width = 1120,                                   \
                plot_height = 500,                                  \
                y_range = [-3,3],                                   \
                x_axis_type = 'datetime',                           \
                x_axis_label = 'Date', y_axis_label = 'mV (normalized)')

    p.grid.visible = True
    p.title.text = "Gratingwheel Y"
    p.title.align = "left"
    pf.add_basic_layout(p)

    pf.add_to_plot_normalized(p, "PRISM", "INRSI_C_GWA_Y_POSITION_PRISM", start, end, conn, mn.gwy_nominals['PRISM'], color = "green")
    pf.add_to_plot_normalized(p, "MIRROR", "INRSI_C_GWA_Y_POSITION_MIRROR", start, end, conn, mn.gwy_nominals['MIRROR'], color = "blue")
    pf.add_to_plot_normalized(p, "G140H", "INRSI_C_GWA_Y_POSITION_G140H", start, end, conn, mn.gwy_nominals['G140H'], color = "red")
    pf.add_to_plot_normalized(p, "G235H", "INRSI_C_GWA_Y_POSITION_G235H", start, end, conn, mn.gwy_nominals['G235H'], color = "purple")
    pf.add_to_plot_normalized(p, "G395H", "INRSI_C_GWA_Y_POSITION_G395H", start, end, conn, mn.gwy_nominals['G395H'], color = "orange")
    pf.add_to_plot_normalized(p, "G140M", "INRSI_C_GWA_Y_POSITION_G140M", start, end, conn, mn.gwy_nominals['G140M'], color = "brown")
    pf.add_to_plot_normalized(p, "G235M", "INRSI_C_GWA_Y_POSITION_G235M", start, end, conn, mn.gwy_nominals['G235M'], color = "darkmagenta")
    pf.add_to_plot_normalized(p, "G395M", "INRSI_C_GWA_Y_POSITION_G395M", start, end, conn, mn.gwy_nominals['G395M'], color = "darkcyan")

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"
    p.legend.orientation = 'horizontal'
    return p

def wheel_pos(conn, start, end):
    '''Combine plots to a tab
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : tab object
        used by dashboard.py to set up dashboard
    '''
    descr = Div(text=
    """
    <style>
    table, th, td {
      border: 1px solid black;
      background-color: #efefef;
      border-collapse: collapse;
      padding: 5px
    }
    table {
      border-spacing: 15px;
    }
    </style>

    <body>
    <table style="width:100%">
      <tr>
        <th><h6>Plotname</h6></th>
        <th><h6>Mnemonic</h6></th>
        <th><h6>Description</h6></th>
      </tr>

      <tr>
        <td>Filterwheel</td>
        <td>INRSI_FWA_MECH_POS<br>
            INRSI_C_FWA_POSITION</td>
        <td>Position Sensor Value<br>
            Current Position</td>
      </tr>

      <tr>
        <td>Gratingwheel X</td>
        <td>INRSI_GWA_MECH_POS<br>
            INRSI_C_GWA_X_POSITION</td>
        <td>Position X Sensor Value<br>
            Current Position</td>
      </tr>

      <tr>
        <td>Gratingwheel Y</td>
        <td>INRSI_GWA_MECH_POS<br>
            INRSI_C_GWA_Y_POSITION</td>
        <td>Position Y Sensor Value<br>
            Current Position</td>
      </tr>

    </table>
    </body>
    """, width=1100)

    plot1 = fw(conn, start, end)
    plot2 = gwx(conn, start, end)
    plot3 = gwy(conn,  start, end)

    layout = column(descr, plot1, plot2, plot3)
    tab = Panel(child = layout, title = "FILTER/GRATINGWHEEL")

    return tab



#! /usr/bin/env python
"""Tests for the ``engineering_database`` module.

Authors
-------

    - Johannes Sahlmann


Use
---

    These tests can be run via the command line (omit the ``-s`` to
    suppress verbose output to ``stdout``):

    ::

        pytest -s test_edb.py
"""

from astropy.time import Time
import pytest


@pytest.mark.xfail(raises=(RuntimeError, FileNotFoundError))
def test_get_mnemonic():
    """Test the query of a single mnemonic."""
    from jwql.edb.engineering_database import get_mnemonic

    mnemonic_identifier = 'IMIR_HK_ICE_SEC_VOLT4'
    start_time = Time('2019-01-16 00:00:00.000', format='iso')
    end_time = Time('2019-01-16 00:01:00.000', format='iso')

    mnemonic = get_mnemonic(mnemonic_identifier, start_time, end_time)
    assert len(mnemonic.data) == mnemonic.meta['paging']['rows']


@pytest.mark.xfail(raises=(RuntimeError, FileNotFoundError))
def test_get_mnemonic_info():
    """Test retrieval of mnemonic info."""
    from jwql.edb.engineering_database import get_mnemonic_info

    mnemonic_identifier = 'IMIR_HK_ICE_SEC_VOLT4'
    info = get_mnemonic_info(mnemonic_identifier)
    assert 'subsystem' in info.keys()


@pytest.mark.xfail(raises=(RuntimeError, FileNotFoundError))
def test_get_mnemonics():
    """Test the query of a list of mnemonics."""
    from jwql.edb.engineering_database import get_mnemonics

    mnemonics = ['SA_ZFGOUTFOV', 'SA_ZFGBADCNT']
    start_time = Time(2018.0, format='decimalyear')
    end_time = Time(2018.1, format='decimalyear')

    mnemonic_dict = get_mnemonics(mnemonics, start_time, end_time)
    assert len(mnemonic_dict) == len(mnemonics)



"""Provides an OAuth object for authentication of the ``jwql`` web app,
as well as decorator functions to require user authentication in other
views of the web application.


Authors
-------

    - Matthew Bourque
    - Christian Mesh

Use
---

    This module is intended to be imported and used as such:
    ::

        from .oauth import auth_info
        from .oauth import auth_required
        from .oauth import JWQL_OAUTH

        @auth_info
        def some_view(request):
            pass

        @auth_required
        def login(request):
            pass

References
----------
    Much of this code was taken from the ``authlib`` documentation,
    found here: ``http://docs.authlib.org/en/latest/client/django.html``

Dependencies
------------
    The user must have a configuration file named ``config.json``
    placed in the ``jwql/utils/`` directory.
"""

import os
import requests

from authlib.django.client import OAuth
from django.shortcuts import redirect, render

import jwql
from jwql.utils.constants import MONITORS
from jwql.utils.utils import get_base_url, get_config

PREV_PAGE = ''


def register_oauth():
    """Register the ``jwql`` application with the ``auth.mast``
    authentication service.

    Returns
    -------
    oauth : Object
        An object containing methods to authenticate a user, provided
        by the ``auth.mast`` service.
    """

    # Get configuration parameters
    client_id = get_config()['client_id']
    client_secret = get_config()['client_secret']
    auth_mast = get_config()['auth_mast']

    # Register with auth.mast
    oauth = OAuth()
    client_kwargs = {'scope': 'mast:user:info'}
    oauth.register(
        'mast_auth',
        client_id='{}'.format(client_id),
        client_secret='{}'.format(client_secret),
        access_token_url='https://{}/oauth/access_token?client_secret={}'.format(auth_mast, client_secret),
        access_token_params=None,
        refresh_token_url=None,
        authorize_url='https://{}/oauth/authorize'.format(auth_mast),
        api_base_url='https://{}/1.1/'.format(auth_mast),
        client_kwargs=client_kwargs)

    return oauth


JWQL_OAUTH = register_oauth()


def authorize(request):
    """Spawn the authentication process for the user

    The authentication process involves retreiving an access token
    from ``auth.mast`` and porting the data to a cookie.

    Parameters
    ----------
    request : HttpRequest object
        Incoming request from the webpage

    Returns
    -------
    HttpResponse object
        Outgoing response sent to the webpage
    """

    # Get auth.mast token
    token = JWQL_OAUTH.mast_auth.authorize_access_token(request, headers={'Accept': 'application/json'})

    # Determine domain
    base_url = get_base_url()
    if '127' in base_url:
        domain = '127.0.0.1'
    else:
        domain = base_url.split('//')[-1]

    # Set secure cookie parameters
    cookie_args = {}
    # cookie_args['domain'] = domain  # Currently broken
    # cookie_args['secure'] = True  # Currently broken
    cookie_args['httponly'] = True

    # Set the cookie
    response = redirect(PREV_PAGE)
    response.set_cookie("ASB-AUTH", token["access_token"], **cookie_args)

    return response


def auth_info(fn):
    """A decorator function that will return user credentials along
    with what is returned by the original function.

    Parameters
    ----------
    fn : function
        The function to decorate

    Returns
    -------
    user_info : function
        The decorated function
    """

    def user_info(request, **kwargs):
        """Store authenticated user credentials in a cookie and return
        it.  If the user is not authenticated, store no credentials in
        the cookie.

        Parameters
        ----------
        request : HttpRequest object
            Incoming request from the webpage

        Returns
        -------
        fn : function
            The decorated function
        """

        cookie = request.COOKIES.get("ASB-AUTH")

        # If user is authenticated, return user credentials
        if cookie is not None:
            response = requests.get(
                'https://{}/info'.format(get_config()['auth_mast']),
                headers={'Accept': 'application/json',
                         'Authorization': 'token {}'.format(cookie)})
            response = response.json()
            response['access_token'] = cookie

        # If user is not authenticated, return no credentials
        else:
            response = {'ezid': None, "anon": True, 'access_token': None}

        return fn(request, response, **kwargs)

    return user_info


def auth_required(fn):
    """A decorator function that requires the given function to have
    authentication through ``auth.mast`` set up.

    Parameters
    ----------
    fn : function
        The function to decorate

    Returns
    -------
    check_auth : function
        The decorated function
    """

    @auth_info
    def check_auth(request, user, **kwargs):
        """Check if the user is authenticated through ``auth.mast``.
        If not, perform the authorization.

        Parameters
        ----------
        request : HttpRequest object
            Incoming request from the webpage
        user : dict
            A dictionary of user credentials

        Returns
        -------
        fn : function
            The decorated function
        """

        # If user is currently anonymous, require a login
        if user['ezid']:

            return fn(request, user, **kwargs)

        else:
            template = 'not_authenticated.html'
            context = {'inst': ''}

            return render(request, template, context)

    return check_auth


@auth_info
def login(request, user):
    """Spawn a login process for the user

    The ``auth_requred`` decorator is used to require that the user
    authenticate through ``auth.mast``, then the user is redirected
    back to the homepage.

    Parameters
    ----------
    request : HttpRequest object
        Incoming request from the webpage
    user : dict
        A dictionary of user credentials.

    Returns
    -------
    HttpResponse object
        Outgoing response sent to the webpage
    """

    # Redirect to oauth login
    global PREV_PAGE
    PREV_PAGE = request.META.get('HTTP_REFERER')
    redirect_uri = os.path.join(get_base_url(), 'authorize')

    return JWQL_OAUTH.mast_auth.authorize_redirect(request, redirect_uri)


def logout(request):
    """Spawn a logout process for the user

    Upon logout, the user's ``auth.mast`` credientials are removed and
    the user is redirected back to the homepage.

    Parameters
    ----------
    request : HttpRequest object
        Incoming request from the webpage
    user : dict
        A dictionary of user credentials.

    Returns
    -------
    HttpResponse object
        Outgoing response sent to the webpage
    """

    global PREV_PAGE
    PREV_PAGE = request.META.get('HTTP_REFERER')
    response = redirect(PREV_PAGE)
    response.delete_cookie("ASB-AUTH")

    return response



#! /usr/bin/env python

"""
Create a preview image from a fits file containing an observation.

This module creates and saves a "preview image" from a fits file that
contains a JWST observation. Data from the user-supplied ``extension``
of the file are read in, along with the ``PIXELDQ`` extension if
present. For each integration in the exposure, the first group is
subtracted from the final group in order to create a difference image.
The lower and upper limits to be displayed are defined as the
``clip_percent`` and ``(1. - clip_percent)`` percentile signals.
``matplotlib`` is then used to display a linear- or log-stretched
version of the image, with accompanying colorbar. The image is then
saved.

Authors:
--------

    - Bryan Hilbert

Use:
----

    This module can be imported as such:

    ::

        from jwql.preview_image.preview_image import PreviewImage
        im = PreviewImage(my_file, "SCI")
        im.clip_percent = 0.01
        im.scaling = 'log'
        im.output_format = 'jpg'
        im.make_image()
"""

import logging
import os
import socket

from astropy.io import fits
import numpy as np

from jwql.utils import permissions

# Use the 'Agg' backend to avoid invoking $DISPLAY
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.colors as colors

# Only import jwst if not running from readthedocs
if 'build' and 'project' not in socket.gethostname():
    from jwst.datamodels import dqflags


class PreviewImage():
    """An object for generating and saving preview images, used by
    ``generate_preview_images``.

    Attributes
    ----------
    clip_percent : float
        The amount to sigma clip the input data by when scaling the
        preview image.  Default is 0.01.
    cmap : str
        The colormap used by ``matplotlib`` in the preview image.
        Default value is ``viridis``.
    data : obj
        The data used to generate the preview image.
    dq : obj
        The DQ data used to generate the preview image.
    file : str
        The filename to generate the preview image from.
    output_format : str
        The format to which the preview image is saved.  Options are
        ``jpg`` and ``thumb``
    preview_output_directory : str or None
        The output directory to which the preview image is saved.
    scaling : str
        The scaling used in the preview image.  Default is ``log``.
    thumbnail_output_directory : str or None
        The output directory to which the thumbnail is saved.

    Methods
    -------
    difference_image(data)
        Create a difference image from the data
    find_limits(data, pixmap, clipperc)
        Find the min and max signal levels after clipping by
        ``clipperc``
    get_data(filename, ext)
        Read in data from the given ``filename`` and ``ext``
    make_figure(image, integration_number, min_value, max_value, scale, maxsize, thumbnail)
        Create the ``matplotlib`` figure
    make_image(max_img_size)
        Main function
    save_image(fname, thumbnail)
        Save the figure
    """

    def __init__(self, filename, extension):
        """Initialize the class.

        Parameters
        ----------
        filename : str
            Name of fits file containing data
        extension : str
            Extension name to be read in
        """
        self.clip_percent = 0.01
        self.cmap = 'viridis'
        self.file = filename
        self.output_format = 'jpg'
        self.preview_output_directory = None
        self.scaling = 'log'
        self.thumbnail_output_directory = None

        # Read in file
        self.data, self.dq = self.get_data(self.file, extension)

    def difference_image(self, data):
        """
        Create a difference image from the data. Use last group minus
        first group in order to maximize signal to noise. With 4D
        input, make a separate difference image for each integration.

        Parameters
        ----------
        data : obj
            4D ``numpy`` ``ndarray`` array of floats

        Returns
        -------
        result : obj
            3D ``numpy`` ``ndarray`` containing the difference image(s)
            from the input exposure
        """
        return data[:, -1, :, :] - data[:, 0, :, :]

    def find_limits(self, data, pixmap, clipperc):
        """
        Find the minimum and maximum signal levels after clipping the
        top and bottom ``clipperc`` of the pixels.

        Parameters
        ----------
        data : obj
            2D numpy ndarray of floats
        pixmap : obj
            2D numpy ndarray boolean array of science pixel locations
            (``True`` for science pixels, ``False`` for non-science
            pixels)
        clipperc : float
            Fraction of top and bottom signal levels to clip (e.g. 0.01
            means to clip brightest and dimmest 1% of pixels)

        Returns
        -------
        results : tuple
            Tuple of floats, minimum and maximum signal levels
        """
        nelem = np.sum(pixmap)
        numclip = np.int(clipperc * nelem)
        sorted = np.sort(data[pixmap], axis=None)
        minval = sorted[numclip]
        maxval = sorted[-numclip - 1]
        return (minval, maxval)

    def get_data(self, filename, ext):
        """
        Read in the data from the given file and extension.  Also find
        how many rows/cols of reference pixels are present.

        Parameters
        ----------
        filename : str
            Name of fits file containing data
        ext : str
            Extension name to be read in

        Returns
        -------
        data : obj
            Science data from file. A 2-, 3-, or 4D numpy ndarray
        dq : obj
            2D ``ndarray`` boolean map of reference pixels. Science
            pixels flagged as ``True`` and non-science pixels are
            ``False``
        """
        if os.path.isfile(filename):
            extnames = []
            with fits.open(filename) as hdulist:
                for exten in hdulist:
                    try:
                        extnames.append(exten.header['EXTNAME'])
                    except:
                        pass
                if ext in extnames:
                    dimensions = len(hdulist[ext].data.shape)
                    if dimensions == 4:
                        data = hdulist[ext].data[:, [0, -1], :, :].astype(np.float)
                    else:
                        data = hdulist[ext].data.astype(np.float)
                else:
                    raise ValueError(('WARNING: no {} extension in {}!'.format(ext, filename)))
                if 'PIXELDQ' in extnames:
                    dq = hdulist['PIXELDQ'].data
                    dq = (dq & dqflags.pixel['NON_SCIENCE'] == 0)
                else:
                    yd, xd = data.shape[-2:]
                    dq = np.ones((yd, xd), dtype="bool")


                # Collect information on aperture location within the
                # full detector. This is needed for mosaicking NIRCam
                # detectors later.
                try:
                    self.xstart = hdulist[0].header['SUBSTRT1']
                    self.ystart = hdulist[0].header['SUBSTRT2']
                    self.xlen = hdulist[0].header['SUBSIZE1']
                    self.ylen = hdulist[0].header['SUBSIZE2']
                except KeyError:
                    logging.warning('SUBSTR and SUBSIZE header keywords not found')

        else:
            raise FileNotFoundError(('WARNING: {} does not exist!'.format(filename)))

        return data, dq

    def make_figure(self, image, integration_number, min_value, max_value,
                    scale, maxsize=8, thumbnail=False):
        """
        Create the matplotlib figure of the image

        Parameters
        ----------
        image : obj
            2D ``numpy`` ``ndarray`` of floats

        integration_number : int
            Integration number within exposure

        min_value : float
            Minimum value for display

        max_value : float
            Maximum value for display

        scale : str
            Image scaling (``log``, ``linear``)

        maxsize : int
            Size of the longest dimension of the output figure (inches)

        thumbnail : bool
            True to create a thumbnail image, False to create the full
            preview image

        Returns
        -------
        result : obj
            Matplotlib Figure object
        """

        # Check the input scaling
        if scale not in ['linear', 'log']:
            raise ValueError(('WARNING: scaling option {} not supported.'.format(scale)))

        # Set the figure size
        yd, xd = image.shape
        ratio = yd / xd
        if xd >= yd:
            xsize = maxsize
            ysize = maxsize * ratio
        else:
            ysize = maxsize
            xsize = maxsize / ratio

        if scale == 'log':

            # Shift data so everything is positive
            shiftdata = image - min_value + 1
            shiftmin = 1
            shiftmax = max_value - min_value + 1

            # If making a thumbnail, make a figure with no axes
            if thumbnail:
                fig = plt.imshow(shiftdata,
                                 norm=colors.LogNorm(vmin=shiftmin,
                                                     vmax=shiftmax),
                                 cmap=self.cmap)
                # Invert y axis
                plt.gca().invert_yaxis()

                plt.axis('off')
                fig.axes.get_xaxis().set_visible(False)
                fig.axes.get_yaxis().set_visible(False)

            # If preview image, add axes and colorbars
            else:
                fig, ax = plt.subplots(figsize=(xsize, ysize))
                cax = ax.imshow(shiftdata,
                                norm=colors.LogNorm(vmin=shiftmin,
                                                    vmax=shiftmax),
                                cmap=self.cmap)
                # Invert y axis
                plt.gca().invert_yaxis()

                # Add colorbar, with original data values
                tickvals = np.logspace(np.log10(shiftmin), np.log10(shiftmax), 5)
                tlabelflt = tickvals + min_value - 1

                # Adjust the number of digits after the decimal point
                # in the colorbar labels based on the signal range
                delta = tlabelflt[-1] - tlabelflt[0]
                if delta >= 100:
                    dig = 0
                elif ((delta < 100) & (delta >= 10)):
                    dig = 1
                elif ((delta < 10) & (delta >= 1)):
                    dig = 2
                elif delta < 1:
                    dig = 3
                format_string = "%.{}f".format(dig)
                tlabelstr = [format_string % number for number in tlabelflt]
                cbar = fig.colorbar(cax, ticks=tickvals)
                cbar.ax.set_yticklabels(tlabelstr)
                cbar.ax.tick_params(labelsize=maxsize * 5. / 4)
                ax.set_xlabel('Pixels', fontsize=maxsize * 5. / 4)
                ax.set_ylabel('Pixels', fontsize=maxsize * 5. / 4)
                ax.tick_params(labelsize=maxsize)
                plt.rcParams.update({'axes.titlesize': 'small'})
                plt.rcParams.update({'font.size': maxsize * 5. / 4})
                plt.rcParams.update({'axes.labelsize': maxsize * 5. / 4})
                plt.rcParams.update({'ytick.labelsize': maxsize * 5. / 4})
                plt.rcParams.update({'xtick.labelsize': maxsize * 5. / 4})

        elif scale == 'linear':
            fig, ax = plt.subplots(figsize=(xsize, ysize))
            cax = ax.imshow(image, clim=(min_value, max_value), cmap=self.cmap)

            if not thumbnail:
                cbar = fig.colorbar(cax)
                ax.set_xlabel('Pixels')
                ax.set_ylabel('Pixels')

        # If preview image, set a title
        if not thumbnail:
            filename = os.path.split(self.file)[-1]
            ax.set_title(filename + ' Int: {}'.format(np.int(integration_number)))

    def make_image(self, max_img_size=8):
        """The main function of the ``PreviewImage`` class."""

        shape = self.data.shape

        if len(shape) == 4:
            # Create difference image(s)
            diff_img = self.difference_image(self.data)
        elif len(shape) < 4:
            diff_img = self.data

        # If there are multiple integrations in the file,
        # work on one integration at a time from here onwards
        ndim = len(diff_img.shape)
        if ndim == 2:
            diff_img = np.expand_dims(diff_img, axis=0)
        nint, ny, nx = diff_img.shape

        for i in range(nint):
            frame = diff_img[i, :, :]

            # Find signal limits for the display
            minval, maxval = self.find_limits(frame, self.dq,
                                              self.clip_percent)

            # Create preview image matplotlib object
            indir, infile = os.path.split(self.file)
            suffix = '_integ{}.{}'.format(i, self.output_format)
            if self.preview_output_directory is None:
                outdir = indir
            else:
                outdir = self.preview_output_directory
            outfile = os.path.join(outdir, infile.split('.')[0] + suffix)
            self.make_figure(frame, i, minval, maxval, self.scaling.lower(),
                             maxsize=max_img_size, thumbnail=False)
            self.save_image(outfile, thumbnail=False)
            plt.close()

            # Create thumbnail image matplotlib object
            if self.thumbnail_output_directory is None:
                outdir = indir
            else:
                outdir = self.thumbnail_output_directory
            outfile = os.path.join(outdir, infile.split('.')[0] + suffix)
            self.make_figure(frame, i, minval, maxval, self.scaling.lower(),
                             maxsize=max_img_size, thumbnail=True)
            self.save_image(outfile, thumbnail=True)
            plt.close()

    def save_image(self, fname, thumbnail=False):
        """
        Save an image in the requested output format and sets the
        appropriate permissions

        Parameters
        ----------
        image : obj
            A ``matplotlib`` figure object

        fname : str
            Output filename

        thumbnail : bool
            True if saving a thumbnail image, false for the full
            preview image.
        """

        plt.savefig(fname, bbox_inches='tight', pad_inches=0)
        permissions.set_permissions(fname)

        # If the image is a thumbnail, rename to '.thumb'
        if thumbnail:
            thumb_fname = fname.replace('.jpg', '.thumb')
            os.rename(fname, thumb_fname)
            logging.info('Saved image to {}'.format(thumb_fname))
        else:
            logging.info('Saved image to {}'.format(fname))



#! /usr/bin/env python
"""Prepares plots for Ref. Voltage/Currents tab

    Module prepares plots for mnemonics below. Combines plots in a grid and
    returns tab object.

    Plot 1 - Ref Voltages
    INRSH_FWA_MOTOR_VREF
    INRSH_GWA_MOTOR_VREF
    INRSH_OA_VREF

    Plot 2 - ADCMGAIN (Voltages)
    INRSH_FWA_ADCMGAIN
    INRSH_GWA_ADCMGAIN
    INRSH_RMA_ADCMGAIN

    Plot 3 - OFFSET (Voltages)
    INRSH_GWA_ADCMOFFSET
    INRSH_FWA_ADCMOFFSET
    INRSH_OA_VREFOFF
    INRSH_RMA_ADCMOFFSET


Authors
-------
    - Daniel Kühbacher

Use
---
    The functions within this module are intended to be imported and
    used by ``nirspec_dashboard.py``, e.g.:

    ::
        from .plots.voltage_tab import voltage_plots
        tab = voltage_plots(conn, start, end)

Dependencies
------------
    User must provide database "nirpsec_database.db"

"""
import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.sql_interface as sql
import jwql.instrument_monitors.nirspec_monitors.data_trending.plots.plot_functions as pf
from bokeh.models import LinearAxis, Range1d
from bokeh.plotting import figure
from bokeh.models.widgets import Panel, Tabs, Div
from bokeh.models import ColumnDataSource, HoverTool
from bokeh.layouts import WidgetBox, gridplot, Column

import pandas as pd
import numpy as np

from astropy.time import Time


def ref_volt(conn, start, end):
    '''Create specific plot and return plot object
    
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''
    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 1120,
                plot_height = 500,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Voltage (V)')

    p.grid.visible = True
    p.title.text = "Ref Voltages"
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "FWA_MOTOR_VREF", "INRSH_FWA_MOTOR_VREF", start, end, conn, color = "green")
    b = pf.add_to_plot(p, "GWA_MOTOR_VREF", "INRSH_GWA_MOTOR_VREF", start, end, conn, color = "blue")
    c = pf.add_to_plot(p, "OA_VREF", "INRSH_OA_VREF", start, end, conn, color = "red")

    pf.add_hover_tool(p,[a,b,c])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"
    p.legend.orientation = "horizontal"
    return p


def gain_volt(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 1120,
                plot_height = 500,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Voltage (V)')

    p.grid.visible = True
    p.title.text = "ADCMAIN"
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "FWA_ADCMGAIN", "INRSH_FWA_ADCMGAIN", start, end, conn, color = "green")
    b = pf.add_to_plot(p, "GWA_ADCMGAIN", "INRSH_GWA_ADCMGAIN", start, end, conn, color = "blue")
    c = pf.add_to_plot(p, "RMA_ADCMGAIN", "INRSH_RMA_ADCMGAIN", start, end, conn, color = "red")


    #pf.add_hover_tool(p,[a,b,c])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"
    p.legend.orientation = "horizontal"
    return p

def offset_volt(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 1120,
                plot_height = 500,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Voltage (V)')

    p.grid.visible = True
    p.title.text = "OFFSET"
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "GWA_ADCMOFFSET", "INRSH_GWA_ADCMOFFSET", start, end, conn, color = "blue")
    b = pf.add_to_plot(p, "FWA_ADCMOFFSET", "INRSH_FWA_ADCMOFFSET", start, end, conn, color = "green")
    c = pf.add_to_plot(p, "OA_VREFOFF", "INRSH_OA_VREFOFF", start, end, conn, color = "orange")
    d = pf.add_to_plot(p, "RMA_ADCMOFFSET", "INRSH_RMA_ADCMOFFSET", start, end, conn, color = "red")

    pf.add_hover_tool(p,[a,b,c,d])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"
    p.legend.orientation = "horizontal"

    return p


def volt_plots(conn, start, end):
    '''Combines plots to a tab
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : tab object
        used by dashboard.py to set up dashboard
    '''
    descr = Div(text=
    """
    <style>
    table, th, td {
      border: 1px solid black;
      background-color: #efefef;
      border-collapse: collapse;
      padding: 5px
    }
    table {
      border-spacing: 15px;
    }
    </style>

    <body>
    <table style="width:100%">
      <tr>
        <th><h6>Plotname</h6></th>
        <th><h6>Mnemonic</h6></th>
        <th><h6>Description</h6></th>
      </tr>
      <tr>
        <td>Ref Voltages</td>
        <td>INRSH_FWA_MOTOR_VREF<br>
            INRSH_GWA_MOTOR_VREF<br>
            INRSH_OA_VREF</td>
        <td>FWA Motor Reference Voltage for Calibration<br>
            GWA Motor Reference Voltage for Calibration<br>
            OA/RMA Reference Voltage for TM Calibration<br>
        </td>
      </tr>

      <tr>
        <td>ADCMGAIN</td>
        <td>INRSH_FWA_ADCMGAIN<br>
            INRSH_GWA_ADCMGAIN<br>
            INRSH_RMA_ADCMGAIN</td>
        <td>FWA ADC Motor Chain Gain for Calibration<br>
            GWA ADC Motor Chain Gain for Calibration<br>
            RMA ADC Motor Chain Gain for Calibration<br>
        </td>
      </tr>

      <tr>
        <td>OFFSET</td>
        <td>INRSH_FWA_ADCMOFFSET<br>
            INRSH_GWA_ADCMOFFSET<br>
            INRSH_OA_VREFOFF<br>
            INRSH_RMA_ADCMOFFSET</td>
        <td>FWA ADC Motor Chain Offset for Calibration<br>
            GWA ADC Motor Chain Offset for Calibration<br>
            CAA Reference Voltage Offset for TM Calibration<br>
            RMA ADC Motor Chain Offset for Calibration<br>
        </td>
      </tr>
    </table>
    </body>
    """, width=1100)

    plot1 = ref_volt(conn, start, end)
    plot2 = gain_volt(conn, start, end)
    plot3 = offset_volt(conn, start, end)

    layout = Column(descr, plot1, plot2, plot3)

    tab = Panel(child = layout, title = "REF VOLTAGES")

    return tab



#! /usr/bin/env python

"""Tests for the ``preview_image`` module.

Authors
-------

    - Johannes Sahlmann
    - Lauren Chambers


Use
---

    These tests can be run via the command line (omit the ``-s`` to
    suppress verbose output to ``stdout``):

    ::

        pytest -s test_preview_image.py
"""

import glob
import os
import pytest
import shutil

from astropy.io import fits

from jwql.utils.preview_image import PreviewImage
from jwql.utils.utils import get_config, ensure_dir_exists

# directory to be created and populated during tests running
TEST_DIRECTORY = os.path.join(os.environ['HOME'], 'preview_image_test')

# Determine if tests are being run on jenkins
ON_JENKINS = os.path.expanduser('~') == '/home/jenkins'


@pytest.fixture(scope="module")
def test_directory(test_dir=TEST_DIRECTORY):
    """Create a test directory for preview image.

    Parameters
    ----------
    test_dir : str
        Path to directory used for testing

    Yields
    -------
    test_dir : str
        Path to directory used for testing

    """
    # Set up local test directory
    ensure_dir_exists(test_dir)
    yield test_dir

    # Tear down local test directory and any files within
    if os.path.isdir(test_dir):
        shutil.rmtree(test_dir)

    # Empty test directory on central storage
    jpgs = glob.glob(os.path.join(get_config()['test_dir'], '*.jpg'))
    thumbs = glob.glob(os.path.join(get_config()['test_dir'], '*.thumbs'))
    for file in jpgs + thumbs:
        os.remove(file)


def get_test_fits_files():
    """Get a list of the FITS files on central storage to make preview images.

    Returns
    -------
    filenames : list
        List of filepaths to FITS files
    """
    # Get the files from central store
    if not ON_JENKINS:
        filenames = glob.glob(os.path.join(get_config()['test_dir'], '*.fits'))
        assert len(filenames) > 0
        return filenames

    # Or return an empty list
    else:
        return []


@pytest.mark.skipif(ON_JENKINS, reason='Requires access to central storage.')
@pytest.mark.parametrize('filename', get_test_fits_files())
def test_make_image(test_directory, filename):
    """Use PreviewImage.make_image to create preview images of a sample
    JWST exposure.

    Assert that the number of JPGs created corresponds to the number of
    integrations.

    Parameters
    ----------
    test_directory : str
        Path of directory used for testing
    filename : str
        Path of FITS image to generate preview of
    """

    header = fits.getheader(filename)

    # Create and save the preview image or thumbnail
    for create_thumbnail in [False, True]:
        try:
            image = PreviewImage(filename, "SCI")
            image.clip_percent = 0.01
            image.scaling = 'log'
            image.cmap = 'viridis'
            image.output_format = 'jpg'
            image.thumbnail = create_thumbnail

            if create_thumbnail:
                image.thumbnail_output_directory = test_directory
            else:
                image.preview_output_directory = test_directory

            image.make_image()
        except ValueError as error:
            print(error)

        if create_thumbnail:
            extension = 'thumb'
        else:
            extension = 'jpg'

        # list of preview images
        preview_image_filenames = glob.glob(os.path.join(test_directory, '*.{}'.format(
            extension)))
        assert len(preview_image_filenames) == header['NINTS']

        # clean up: delete preview images
        for file in preview_image_filenames:
            os.remove(file)



#! /usr/bin/env python
"""Prepares plots for TEMPERATURE tab

    Module prepares plots for mnemonics below. Combines plots in a grid and
    returns tab object.

    Plot 1:
    IGDP_MIR_ICE_T1P_CRYO
    IGDP_MIR_ICE_T2R_CRYO
    IGDP_MIR_ICE_T3LW_CRYO
    IGDP_MIR_ICE_T4SW_CRYO
    IGDP_MIR_ICE_T5IMG_CRYO
    IGDP_MIR_ICE_T6DECKCRYO
    IGDP_MIR_ICE_T7IOC_CRYO
    IGDP_MIR_ICE_FW_CRYO
    IGDP_MIR_ICE_CCC_CRYO
    IGDP_MIR_ICE_GW14_CRYO
    IGDP_MIR_ICE_GW23_CRYO
    IGDP_MIR_ICE_POMP_CRYO
    IGDP_MIR_ICE_POMR_CRYO
    IGDP_MIR_ICE_IFU_CRYO
    IGDP_MIR_ICE_IMG_CRYO

    Plot 2:
    ST_ZTC1MIRIA
    ST_ZTC2MIRIA
    IMIR_PDU_TEMP
    IMIR_IC_SCE_ANA_TEMP1
    IMIR_SW_SCE_ANA_TEMP1
    IMIR_LW_SCE_ANA_TEMP1
    IMIR_IC_SCE_DIG_TEMP
    IMIR_SW_SCE_DIG_TEMP
    IMIR_LW_SCE_DIG_TEMP

    Plot 3:
    IGDP_MIR_IC_DET_TEMP
    IGDP_MIR_LW_DET_TEMP
    IGDP_MIR_SW_DET_TEMP

Authors
-------
    - Daniel Kühbacher

Use
---
    The functions within this module are intended to be imported and
    used by ``dashborad.py``, e.g.:

    ::
        from .plots.temperature_tab import temperature_plots
        tab = temperature_plots(conn, start, end)

Dependencies
------------
    User must provide database "miri_database.db"

"""
import jwql.instrument_monitors.miri_monitors.data_trending.utils.sql_interface as sql
import jwql.instrument_monitors.miri_monitors.data_trending.plots.plot_functions as pf
from bokeh.plotting import figure
from bokeh.models import BoxAnnotation, LinearAxis, Range1d
from bokeh.embed import components
from bokeh.models.widgets import Panel, Tabs, Div
from bokeh.models import ColumnDataSource
from bokeh.layouts import column

import pandas as pd
import numpy as np

from astropy.time import Time


def cryo(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",       \
                toolbar_location = "above",                         \
                plot_width = 1120,                                   \
                plot_height = 700,                                  \
                y_range = [5.8,6.4],                                \
                x_axis_type = 'datetime',                           \
                output_backend="webgl",                             \
                x_axis_label = 'Date', y_axis_label = 'Temperature (K)')

    p.grid.visible = True
    p.title.text = "Cryo Temperatures"
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "T1P", "IGDP_MIR_ICE_T1P_CRYO", start, end, conn, color = "brown")
    b = pf.add_to_plot(p, "T2R", "IGDP_MIR_ICE_T2R_CRYO", start, end, conn, color = "burlywood")
    c = pf.add_to_plot(p, "T3LW", "IGDP_MIR_ICE_T3LW_CRYO", start, end, conn, color = "cadetblue")
    d = pf.add_to_plot(p, "T4SW", "IGDP_MIR_ICE_T4SW_CRYO", start, end, conn, color = "chartreuse")
    e = pf.add_to_plot(p, "T5IMG", "IGDP_MIR_ICE_T5IMG_CRYO", start, end, conn, color = "chocolate")
    f = pf.add_to_plot(p, "T6DECK", "IGDP_MIR_ICE_T6DECKCRYO", start, end, conn, color = "coral")
    g = pf.add_to_plot(p, "T7IOC", "IGDP_MIR_ICE_T7IOC_CRYO", start, end, conn, color = "darkorange")
    h = pf.add_to_plot(p, "FW", "IGDP_MIR_ICE_FW_CRYO", start, end, conn, color = "crimson")
    i = pf.add_to_plot(p, "CCC", "IGDP_MIR_ICE_CCC_CRYO", start, end, conn, color = "cyan")
    j = pf.add_to_plot(p, "GW14", "IGDP_MIR_ICE_GW14_CRYO", start, end, conn, color = "darkblue")
    k = pf.add_to_plot(p, "GW23", "IGDP_MIR_ICE_GW23_CRYO", start, end, conn, color = "darkgreen")
    l = pf.add_to_plot(p, "POMP", "IGDP_MIR_ICE_POMP_CRYO", start, end, conn, color = "darkmagenta")
    m = pf.add_to_plot(p, "POMR", "IGDP_MIR_ICE_POMR_CRYO", start, end, conn, color = "darkcyan")
    n = pf.add_to_plot(p, "IFU", "IGDP_MIR_ICE_IFU_CRYO", start, end, conn, color = "cornflowerblue")
    o = pf.add_to_plot(p, "IMG", "IGDP_MIR_ICE_IMG_CRYO", start, end, conn, color = "orange")

    pf.add_hover_tool(p,[a,b,c,d,e,f,g,h,i,j,k,l,m,n,o])

    p.legend.location = "bottom_right"
    p.legend.orientation = "horizontal"
    p.legend.click_policy = "hide"

    return p

def temp(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    start_str = str(Time(start).mjd)
    end_str = str(Time(end).mjd)

    sql_c = "SELECT * FROM IGDP_MIR_ICE_INTER_TEMP WHERE start_time BETWEEN "+start_str+" AND "+end_str+" ORDER BY start_time"
    temp = pd.read_sql_query(sql_c, conn)

    temp['average']+= 273.15
    reg = pd.DataFrame({'reg' : pf.pol_regression(temp['start_time'], temp['average'],3)})
    temp = pd.concat([temp, reg], axis=1)

    temp['start_time'] = pd.to_datetime( Time(temp['start_time'], format = "mjd").datetime )
    plot_data = ColumnDataSource(temp)

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",       \
                toolbar_location = "above",                         \
                plot_width = 1120,                                   \
                plot_height = 700,                                  \
                y_range = [275,295],                             \
                x_axis_type = 'datetime',                           \
                output_backend="webgl",                             \
                x_axis_label = 'Date', y_axis_label = 'Temperature (K)')

    p.grid.visible = True
    p.title.text = "IEC Temperatures"
    pf.add_basic_layout(p)

    p.line(x = "start_time", y = "average", color = "brown", legend = "ICE Internal", source = plot_data)
    p.scatter(x = "start_time", y = "average", color = "brown", legend = "ICE Internal", source = plot_data)

    a = pf.add_to_plot(p, "ICE IEC A", "ST_ZTC1MIRIA", start, end, conn, color = "burlywood")
    b = pf.add_to_plot(p, "FPE IEC A", "ST_ZTC2MIRIA", start, end, conn, color = "cadetblue")
    j = pf.add_to_plot(p, "ICE IEC B", "ST_ZTC1MIRIB", start, end, conn, color = "blue")
    k = pf.add_to_plot(p, "FPE IEC B.", "ST_ZTC2MIRIB", start, end, conn, color = "brown")
    c = pf.add_to_plot(p, "FPE PDU", "IMIR_PDU_TEMP", start, end, conn, color = "chartreuse")
    d = pf.add_to_plot(p, "ANA IC", "IMIR_IC_SCE_ANA_TEMP1", start, end, conn, color = "chocolate")
    e = pf.add_to_plot(p, "ANA SW", "IMIR_SW_SCE_ANA_TEMP1", start, end, conn, color = "coral")
    f = pf.add_to_plot(p, "ANA LW", "IMIR_LW_SCE_ANA_TEMP1", start, end, conn, color = "darkorange")
    g = pf.add_to_plot(p, "DIG IC", "IMIR_IC_SCE_DIG_TEMP", start, end, conn, color = "crimson")
    h = pf.add_to_plot(p, "DIG SW", "IMIR_SW_SCE_DIG_TEMP", start, end, conn, color = "cyan")
    i = pf.add_to_plot(p, "DIG LW", "IMIR_LW_SCE_DIG_TEMP", start, end, conn, color = "darkblue")

    pf.add_hover_tool(p,[a,b,c,d,e,f,g,h,i,j,k])

    p.legend.location = "bottom_right"
    p.legend.orientation = "horizontal"
    p.legend.click_policy = "hide"

    return p

def det(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",       \
                toolbar_location = "above",                         \
                plot_width = 1120,                                   \
                plot_height = 400,                                  \
                y_range = [6.395,6.41],                             \
                x_axis_type = 'datetime',                           \
                output_backend="webgl",                             \
                x_axis_label = 'Date', y_axis_label = 'Temperature (K)')

    p.grid.visible = True
    p.title.text = "Detector Temperature"
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "Det. Temp. IC", "IGDP_MIR_IC_DET_TEMP", start, end, conn, color = "red")
    b = pf.add_to_plot(p, "Det. Temp. LW", "IGDP_MIR_LW_DET_TEMP", start, end, conn, color = "green")
    c = pf.add_to_plot(p, "Det. Temp. SW", "IGDP_MIR_SW_DET_TEMP", start, end, conn, color = "blue")

    pf.add_hover_tool(p,[a,b,c])

    p.legend.location = "bottom_right"
    p.legend.orientation = "horizontal"
    p.legend.click_policy = "hide"

    return p

def temperature_plots(conn, start, end):
    '''Combines plots to a tab
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : tab object
        used by dashboard.py to set up dashboard
    '''

    descr = Div(text=
    """
    <style>
    table, th, td {
      border: 1px solid black;
      background-color: #efefef;
      border-collapse: collapse;
      padding: 5px
    }
    </style>

    <body>
    <table style="width:100%">
      <tr>
        <th><h6>Plotname</h6></th>
        <th><h6>Mnemonic</h6></th>
        <th><h6>Description</h6></th>
      </tr>
      <tr>
        <td>CRYO Temperatures</td>
        <td>IGDP_MIR_ICE_T1P_CRYO<br>
            IGDP_MIR_ICE_T2R_CRYO<br>
            IGDP_MIR_ICE_T3LW_CRYO<br>
            IGDP_MIR_ICE_T4SW_CRYO<br>
            IGDP_MIR_ICE_T5IMG_CRYO<br>
            IGDP_MIR_ICE_T6DECKCRYO<br>
            IGDP_MIR_ICE_T7IOC_CRYO<br>
            IGDP_MIR_ICE_FW_CRYO<br>
            IGDP_MIR_ICE_CCC_CRYO<br>
            IGDP_MIR_ICE_GW14_CRYO<br>
            IGDP_MIR_ICE_GW23_CRYO<br>
            IGDP_MIR_ICE_POMP_CRYO<br>
            IGDP_MIR_ICE_POMR_CRYO<br>
            IGDP_MIR_ICE_IFU_CRYO<br>
            IGDP_MIR_ICE_IMG_CRYO<br></td>
        <td>Deck Nominal Temperature (T1)<br>
            Deck Redundant Temperature (T2)<br>
            LW FPM I/F Temperature (T3)<br>
            SW FPM I/F Temperature (T4)<br>
            IM FPM I/F Temperature (T5)<br>
            A-B Strut Apex Temperature (T6)<br>
            IOC Temperature (T7)<br>
            FWA Temperature<br>
            CCC Temperature<br>
            DGA-A (GW14) Temperature<br>
            DGA-B (GW23) Temperature<br>
            POMH Nominal Temperature<br>
            POMH Redundant Temperature<br>
            MRS (CF) Cal. Source Temperature<br>
            Imager (CI) Cal. Source Temperature<br></td>
      </tr>
      <tr>
        <td>IEC Temperatures</td>
        <td>ST_ZTC1MIRIA<br>
            ST_ZTC2MIRIA<br>
            ST_ZTC1MIRIB<br>
            ST_ZTC2MIRIB<br>
            IGDP_MIR_ICE_INTER_TEMP<br>
            IMIR_PDU_TEMP<br>
            IMIR_IC_SCE_ANA_TEMP1<br>
            IMIR_SW_SCE_ANA_TEMP1<br>
            IMIR_LW_SCE_ANA_TEMP1<br>
            IMIR_IC_SCE_DIG_TEMP<br>
            IMIR_SW_SCE_DIG_TEMP<br>
            IMIR_LW_SCE_DIG_TEMP<br></td>
        <td>ICE IEC Panel Temp A<br>
            FPE IEC Panel Temp A<br>
            ICE IEC Panel Temp B<br>
            FPE IEC Panel Temp B<br>
            ICE internal Temperature<br>
            FPE PDU Temperature<br>
            FPE SCE Analogue board Temperature IC<br>
            FPE SCE Analogue board Temperature SW<br>
            FPE SCE Analogue board Temperature LW<br>
            FPE SCE Digital board Temperature IC<br>
            FPE SCE Digital board Temperature SW<br>
            FPE SCE Digital board Temperature LW<br></td>
      </tr>
       <tr>
         <td>Detector Temperatures</td>
         <td>IGDP_MIR_IC_DET_TEMP<br>
            IGDP_MIR_lW_DET_TEMP<br>
            IGDP_MIR_SW_DET_TEMP<br></td>
         <td>Detector Temperature (IC,SW&LW)<br></td>
       </tr>
    </table>
    </body>
    """, width=1100)


    plot1 = cryo(conn, start, end)
    plot2 = temp(conn, start, end)
    plot3 = det(conn, start, end)

    layout = column(descr, plot1, plot2, plot3)
    tab = Panel(child = layout, title = "TEMPERATURE")

    return tab



"""This module holds functions for miri data trending

All functions in this module are tailored for the miri datatrending application.
Detailed descriptions are given for every function individually.

-------
    - Daniel Kühbacher

Use
---

Dependencies
------------
MIRI_trend_requestsDRAFT1900201.docx

References
----------

Notes
-----

"""

import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.mnemonics as mn
import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.condition as cond
import statistics
import sqlite3
import warnings
import numpy as np
from collections import defaultdict


def extract_data(condition, mnemonic):
    '''Function extracts data from given mnemmonic at a given condition
    Parameters
    ----------
    condition : object
        conditon object that holds one or more subconditions
    mnemonic : AstropyTable
        holds single table with mnemonic data
    Return
    ------
    temp : list  or None
        holds data that applies to given condition
    '''
    temp = []

    #look for all values that fit to the given conditions
    for element in mnemonic:
        if condition.state(float(element['time'])):
            temp.append(float(element['value']))

    #return temp is one ore more values fit to the condition
    #return None if no applicable data was found
    if len(temp) > 0:
        return temp
    else:
        return None

def lamp_distinction(caa_flag, lamp_sel, lamp_curr, lamp_volt):
    """Distincts over all calibration lamps and returns representative current means
        each
    Parameters
    ----------
    """

    #initilize empty dict
    lamp_values = defaultdict(list)

    for index, flag in enumerate(caa_flag):

        if flag['value'] == 'ON':

            #initialize lamp value to default
            current_lamp = "default"

            #find current lamp value
            for lamp in lamp_sel:
                if lamp['time'] <= flag['time']:
                    current_lamp = lamp['value']

            #go to next Value if dummy lamps are activated
            if (current_lamp == 'NO_LAMP') or (current_lamp == 'DUMMY'):
                continue

            #define on_time of current lamp
            try:
                start_time = flag['time']

                i = 1
                if caa_flag[index+i]['value'] == 'OFF':
                    end_time = caa_flag[index+1]['time']
                else:
                    i += 1

            except IndexError:
                break

            #append and evaluate current and voltage values
            temp_curr = []
            temp_volt = []

            #append current values to list
            for curr in lamp_curr:
                if curr['time'] >= start_time:
                    if curr['time'] < end_time:
                        temp_curr.append(float(curr['value']))
                    else:
                        break
            #append voltage values to list
            for volt in lamp_volt:
                if volt['time'] >= start_time :
                    if volt['time'] < end_time:
                        temp_volt.append(float(volt['value']))
                    else:
                        break

            lamp_data = []
            #append current values
            lamp_data.append(start_time)
            lamp_data.append(end_time)
            lamp_data.append(len(temp_curr))
            lamp_data.append(statistics.mean(temp_curr))
            lamp_data.append(statistics.stdev(temp_curr))
            #append voltage values
            lamp_data.append(len(temp_volt))
            lamp_data.append(statistics.mean(temp_volt))
            lamp_data.append(statistics.stdev(temp_volt))
            lamp_values[current_lamp].append(( lamp_data ))

    return lamp_values

def extract_filterpos(move_stat, wheel_pos, wheel_val):
    '''Extracts ratio values which correspond to given position values and their
       proposed nominals
    Parameters
    ----------
    condition : object
        conditon object that holds one or more subconditions
    nominals : dict
        holds nominal values for all wheel positions
    ratio_mem : AstropyTable
        holds ratio values of one specific mnemonic
    pos_mem : AstropyTable
        holds pos values of one specific mnemonic
    Return
    ------
    pos_values : dict
        holds ratio values and times with corresponding positionlabel as key
    '''

    #initilize empty dict for assigned ratio values
    pos_values = defaultdict(list)

    for index, stat in enumerate(move_stat):

        #raise warning if position is UNKNOWN
        if stat['value'] == "SUCCESS":

            #initialize lamp value to default
            current_pos = "default"
            pos_val = 0
            pos_time = 0

            #Evaluate current position
            for pos in wheel_pos:
                if pos['time'] <= stat['time']:
                    current_pos = pos['value']
                if pos['time'] > stat['time']:
                    break

            #Evaluate corresponding value
            for val in wheel_val:
                if val['time'] <= stat['time']:
                    pos_val = val['value']
                    pos_time = val['time']
                if val['time'] > stat['time']:
                    break

            print (current_pos, pos_val, pos_time)

            if current_pos != 'default':
                pos_values[current_pos].append((pos_time, pos_val))
        else:
            continue

    return pos_values

def once_a_day_routine(mnemonic_data):
    '''Routine for processing a 15min data file once a day
    Parameters
    ----------
    mnemonic_data : dict
        dict holds time and value in a astropy table with correspining identifier as key
    Return
    ------
    return_data : dict
        Holds extracted data with applied conditions
    '''

    #abbreviate attribute
    m = mnemonic_data
    return_data = dict()

    ###########################################################################
    con_set_1 = [                                                           \
    cond.unequal(m.mnemonic('INRSD_EXP_STAT'),'STARTED')]
    #setup condition
    condition_1 = cond.condition(con_set_1)

    for identifier in mn.mnemonic_cond_1:
        data = extract_data(condition_1, m.mnemonic(identifier))
        if data != None:
            return_data.update( {identifier:data} )
        else:
            print("no data for {}".format(identifier))
    del condition_1

    ###########################################################################
    con_set_2 = [                                                           \
    cond.equal(m.mnemonic('INRSH_LAMP_SEL'), 'NO_LAMP')]
    #setup condition
    condition_2 = cond.condition(con_set_2)

    for identifier in mn.mnemonic_cond_2:
        data = extract_data(condition_2, m.mnemonic(identifier))
        if data != None:
            return_data.update( {identifier:data} )
        else:
            print("no data for {}".format(identifier))
    del condition_2

    ###########################################################################
    con_set_3 = [                                                           \
    cond.unequal(m.mnemonic('INRSM_MOVE_STAT'), 'STARTED')]
    #setup condition
    condition_3 = cond.condition(con_set_3)

    for identifier in mn.mnemonic_cond_3:
        data = extract_data(condition_3, m.mnemonic(identifier))
        if data != None:
            return_data.update( {identifier:data} )
        else:
            print("no data for {}".format(identifier))
    del condition_3

    return return_data


def whole_day_routine(mnemonic_data):
    '''Proposed routine for processing a 15min data file once a day

    Parameters
    ----------
    mnemonic_data : dict
        dict holds time and value in a astropy table with correspining identifier as key

    Return
    ------
    data_cond_1 : dict
        holds extracted data with condition 1 applied
    data_cond_1 : dict
        holds extracted data with condition 2 applied
    '''

    #abbreviate attribute
    m = mnemonic_data
    return_data = dict()

    ###########################################################################
    con_set_ft_10 = [
    cond.equal(m.mnemonic('ICTM_RT_FILTER'), 10, stringval = False)]
    #setup condition
    condition_ft_10 = cond.condition(con_set_ft_10)

    for identifier in mn.mnemonic_ft10:
        data = extract_data(condition_ft_10, m.mnemonic(identifier))
        if data != None:
            return_data.update( {identifier:data} )
        else:
            print("no data for {}".format(identifier))
    del condition_ft_10

    ##########################################################################
    con_set_caa = [                                                           \
    cond.equal(m.mnemonic('INRSH_CAA_PWRF_ST'), 'ON')]
    #setup condition
    condition_caa = cond.condition(con_set_caa)

    for identifier in mn.mnemonic_caa:
        data = extract_data(condition_caa, m.mnemonic(identifier))

        if data != None:
            return_data.update( {identifier:data} )
        else:
            print("no data for {}".format(identifier))

    del condition_caa

    ###########################################################################
    data_lamps = lamp_distinction(  m.mnemonic('INRSI_CAA_ON_FLAG'),
                                    m.mnemonic('INRSH_LAMP_SEL'),
                                    m.mnemonic('INRSI_C_CAA_CURRENT'),
                                    m.mnemonic('INRSI_C_CAA_VOLTAGE') )

    return return_data, data_lamps

def wheelpos_routine(mnemonic_data):
    '''Proposed routine for positionsensors each day
    Parameters
    ----------
    mnemonic_data : dict
        dict holds time and value in a astropy table with correspining identifier as key
    Return
    ------
    FW : dict
        holds FW ratio values and times with corresponding positionlabel as key
    GW14 : dict
        holds GW14 ratio values and times with corresponding positionlabel as key
    GW23 : dict
        holds GW23 ratio values and times with corresponding positionlabel as key
    CCC : dict
        holds CCC ratio values and times with corresponding positionlabel as key
    '''

    #abbreviate attribute
    m = mnemonic_data

    FW = extract_filterpos( m.mnemonic('INRSI_FWA_MOVE_ST'),
                            m.mnemonic('INRSI_FWA_MECH_POS'),
                            m.mnemonic('INRSI_C_FWA_POSITION'))

    GWX = extract_filterpos(m.mnemonic('INRSI_GWA_MOVE_ST'),
                            m.mnemonic('INRSI_GWA_MECH_POS'),
                            m.mnemonic('INRSI_C_GWA_X_POSITION'))

    GWY = extract_filterpos(m.mnemonic('INRSI_GWA_MOVE_ST'),
                            m.mnemonic('INRSI_GWA_MECH_POS'),
                            m.mnemonic('INRSI_C_GWA_Y_POSITION'))

    return FW, GWX, GWY

if __name__ =='__main__':
    pass



#! /usr/bin/env python

"""Utility module for administrative tasks.

A python script version of Django's command-line utility for
administrative tasks (``django-admin``). Additionally, puts the project
package on ``sys.path`` and defines the ``DJANGO_SETTINGS_MODULE``
variable to point to the jwql ``settings.py`` file.

Generated by ``django-admin startproject`` using Django 2.0.1.

Use
---

    To run the web app server:
    ::

        python manage.py runserver

    To start the interactive shellL:
    ::

        python manage.py shell

    To run tests for all installed apps:
    ::

        python manage.py test

References
----------
For more information please see:
    ``https://docs.djangoproject.com/en/2.0/ref/django-admin/``
"""

import os
import sys

if __name__ == "__main__":

    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "jwql_proj.settings")

    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    execute_from_command_line(sys.argv)



import os
import pkg_resources

module_path = pkg_resources.resource_filename('jwql', '')
setup_path = os.path.normpath(os.path.join(module_path, '../setup.py'))

try:
    with open(setup_path) as f:
        data = f.readlines()

    for line in data:
        if 'VERSION =' in line:
            __version__ = line.split(' ')[-1].replace("'", "").strip()

except FileNotFoundError:
    print('Could not determine jwql version')
    __version__ = '0.0.0'



import statistics
import os
import glob
import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.mnemonics as mn
import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.sql_interface as sql
import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.csv_to_AstropyTable as apt
from jwql.utils.utils import get_config, filename_parser

from astropy.table import Table, Column

from jwql.instrument_monitors.nirspec_monitors.data_trending.utils.process_data import once_a_day_routine

__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))

#point to the directory where your files are located!
directory = os.path.join(get_config()['outputs'], 'nirspec_data_trending', 'nirspec_new_15min', '*.CSV')

#here some some files contain the same data but they are all incomplete
#in order to generate a full database we have to import all of them
filenames = glob.glob(directory)

def process_file(conn, path):
    '''Parse CSV file, process data within and put to DB
    Parameters
    ----------
    conn : DBobject
        Connection object to temporary database
    path : str
        defines path to the files
    '''

    #import mnemonic data and append dict to variable below
    m_raw_data = apt.mnemonics(path)

    #process raw data with once a day routine
    returndata = once_a_day_routine(m_raw_data)

    #put all data in a database that uses a condition
    for key, value in returndata.items():
        m = m_raw_data.mnemonic(key)
        length = len(value)
        mean = statistics.mean(value)
        deviation = statistics.stdev(value)
        dataset = (float(m.meta['start']), float(m.meta['end']), length, mean, deviation)
        sql.add_data(conn, key, dataset)

    #add rest of the data to database
    for identifier in mn.mnemSet_15min:

        m = m_raw_data.mnemonic(identifier)

        temp = []

        #look for all values that fit to the given conditions
        for element in m:
            temp.append(float(element['value']))

        #return None if no applicable data was found
        if len(temp) > 2:
            length = len(temp)
            mean = statistics.mean(temp)
            deviation = statistics.stdev(temp)

            dataset = (float(m.meta['start']), float(m.meta['end']), length, mean, deviation)
            sql.add_data(conn, identifier, dataset)
        elif len(temp) == 2:
            dataset = (float(element['time']), float(element['time']), 1, temp[0], 0)
            sql.add_data(conn, identifier, dataset)
        else:
            print('No data for {}'.format(identifier))
            print(temp)

        del temp


def main():
    #generate paths
    DATABASE_LOCATION = os.path.join(get_config()['jwql_dir'], 'database')
    DATABASE_FILE = os.path.join(DATABASE_LOCATION, 'nirspec_database.db')

    #connect to temporary database
    conn = sql.create_connection(DATABASE_FILE)

    #do for every file in list above
    for path in filenames:
        process_file(conn, path)

    #close connection
    sql.close_connection(conn)
    print("done")

if __name__ == "__main__":
    main()



""" Logging functions for the ``jwql`` automation platform.

This module provides decorators to log the execution of modules.  Log
files are written to the ``logs/`` directory in the ``jwql`` central
storage area, named by module name and timestamp, e.g.
``monitor_filesystem/monitor_filesystem_2018-06-20-15:22:51.log``


Authors
-------

    - Catherine Martlin
    - Alex Viana (WFC3 QL Version)
    - Matthew Bourque

Use
---

    To log the execution of a module, use:
    ::

        import os
        import logging

        from jwql.logging.logging_functions import configure_logging
        from jwql.logging.logging_functions import log_info
        from jwql.logging.logging_functions import log_fail

        @log_info
        @log_fail
        def my_main_function():
            pass

        if __name__ == '__main__':

            module = os.path.basename(__file__).replace('.py', '')
            configure_logging(module)

            my_main_function()

Dependencies
------------

    The user must have a configuration file named ``config.json``
    placed in the ``utils`` directory and it must contain keys for
    ``log_dir`` and ``admin_account``.

References
----------
    This code is adopted and updated from python routine
    ``logging_functions.py`` written by Alex Viana, 2013 for the WFC3
    Quicklook automation platform.
"""

import datetime
import getpass
import importlib
import logging
import os
import pwd
import socket
import sys
import time
import traceback

from functools import wraps

from jwql.utils.permissions import set_permissions
from jwql.utils.utils import get_config, ensure_dir_exists


def configure_logging(module):
    """Configure the log file with a standard logging format.

    Parameters
    ----------
    module : str
        The name of the module being logged.
    production_mode : bool
        Whether or not the output should be written to the production
        environement.
    path : str
        Where to write the log if user-supplied path; default to working dir.

    Returns
    -------
    log_file : str
        The path to the file where the log is written to.
    """

    # Determine log file location
    log_file = make_log_file(module)

    # Make sure no other root lhandlers exist before configuring the logger
    for handler in logging.root.handlers[:]:
        logging.root.removeHandler(handler)

    # Create the log file and set the permissions
    logging.basicConfig(filename=log_file,
                        format='%(asctime)s %(levelname)s: %(message)s',
                        datefmt='%m/%d/%Y %H:%M:%S %p',
                        level=logging.INFO)
    print('Log file initialized to {}'.format(log_file))
    set_permissions(log_file)

    return log_file


def make_log_file(module):
    """Create the log file name based on the module name.

    The name of the ``log_file`` is a combination of the name of the
    module being logged and the current datetime.

    Parameters
    ----------
    module : str
        The name of the module being logged.
    production_mode : bool
        Whether or not the output should be written to the production
        environment.
    path : str
        Where to write the log if user-supplied path; default to
        working dir.

    Returns
    -------
    log_file : str
        The full path to where the log file will be written to.
    """

    # Build filename
    timestamp = datetime.datetime.now().strftime('%Y-%m-%d-%H-%M')
    filename = '{0}_{1}.log'.format(module, timestamp)

    # Determine save location
    user = pwd.getpwuid(os.getuid()).pw_name
    admin_account = get_config()['admin_account']
    log_path = get_config()['log_dir']

    # For production
    if user == admin_account and socket.gethostname()[0] == 'p':
        log_file = os.path.join(log_path, 'prod', module, filename)

    # For test
    elif user == admin_account and socket.gethostname()[0] == 't':
        log_file = os.path.join(log_path, 'test', module, filename)

    # For dev
    elif user == admin_account and socket.gethostname()[0] == 'd':
        log_file = os.path.join(log_path, 'dev', module, filename)

    # For local (also write to dev)
    else:
        log_file = os.path.join(log_path, 'dev', module, filename)

    # Make sure parent directory exists
    ensure_dir_exists(os.path.dirname(log_file))

    return log_file


def log_info(func):
    """Decorator to log useful system information.

    This function can be used as a decorator to log user environment
    and system information. Future packages we want to track can be
    added or removed as necessary.

    Parameters
    ----------
    func : func
        The function to decorate.

    Returns
    -------
    wrapped : func
        The wrapped function.
    """

    @wraps(func)
    def wrapped(*args, **kwargs):

        # Log environment information
        logging.info('User: ' + getpass.getuser())
        logging.info('System: ' + socket.gethostname())
        logging.info('Python Version: ' + sys.version.replace('\n', ''))
        logging.info('Python Executable Path: ' + sys.executable)

        # Read in setup.py file to build list of required modules
        with open(get_config()['setup_file']) as f:
            data = f.readlines()

        for i, line in enumerate(data):
            if 'REQUIRES = [' in line:
                begin = i + 1
            elif 'setup(' in line:
                end = i - 2
        required_modules = data[begin:end]

        # Clean up the module list
        module_list = [item.strip().replace("'", "").replace(",", "").split("=")[0].split(">")[0].split("<")[0] for item in required_modules]

        # Log common module version information
        for module in module_list:
            try:
                mod = importlib.import_module(module)
                logging.info(module + ' Version: ' + mod.__version__)
                logging.info(module + ' Path: ' + mod.__path__[0])
            except (ImportError, AttributeError) as err:
                logging.warning(err)

        logging.info('')

        # Call the function and time it
        t1_cpu = time.clock()
        t1_time = time.time()
        func(*args, **kwargs)
        t2_cpu = time.clock()
        t2_time = time.time()

        # Log execution time
        hours_cpu, remainder_cpu = divmod(t2_cpu - t1_cpu, 60 * 60)
        minutes_cpu, seconds_cpu = divmod(remainder_cpu, 60)
        hours_time, remainder_time = divmod(t2_time - t1_time, 60 * 60)
        minutes_time, seconds_time = divmod(remainder_time, 60)
        logging.info('Elapsed Real Time: {}:{}:{}'.format(int(hours_time), int(minutes_time), int(seconds_time)))
        logging.info('Elapsed CPU Time: {}:{}:{}'.format(int(hours_cpu), int(minutes_cpu), int(seconds_cpu)))

    return wrapped


def log_fail(func):
    """Decorator to log crashes in the decorated code.

    Parameters
    ----------
    func : func
        The function to decorate.

    Returns
    -------
    wrapped : func
        The wrapped function.
    """

    @wraps(func)
    def wrapped(*args, **kwargs):

        try:

            # Run the function
            func(*args, **kwargs)
            logging.info('Completed Successfully')

        except Exception:
            logging.critical(traceback.format_exc())
            logging.critical('CRASHED')

    return wrapped


def log_timing(func):
    """Decorator to time a module or function within a code.

    Parameters
    ----------
    func : func
        The function to time.

    Returns
    -------
    wrapped : func
        The wrapped function. Will log the time."""

    def wrapped(*args, **kwargs):

        # Call the function and time it
        t1_cpu = time.process_time()
        t1_time = time.time()
        func(*args, **kwargs)
        t2_cpu = time.process_time()
        t2_time = time.time()

        # Log execution time
        hours_cpu, remainder_cpu = divmod(t2_cpu - t1_cpu, 60 * 60)
        minutes_cpu, seconds_cpu = divmod(remainder_cpu, 60)
        hours_time, remainder_time = divmod(t2_time - t1_time, 60 * 60)
        minutes_time, seconds_time = divmod(remainder_time, 60)
        logging.info('Elapsed Real Time of {}: {}:{}:{}'.format(func.__name__, int(hours_time), int(minutes_time), int(seconds_time)))
        logging.info('Elapsed CPU Time of {}: {}:{}:{}'.format(func.__name__, int(hours_cpu), int(minutes_cpu), int(seconds_cpu)))

    return wrapped



"""Module lists all neccessary mnemonics for NIRSpec data trending

The module includes several lists to import to NIRSpec data trending monitor program.
The lists are used for data aquisation and to set up the initial database.

Authors
-------
    - Daniel Kühbacher

Use
---
    import mnemoncis as mn

References
----------
    JWQL_NIRSpec_INputs_V4[2414].xlsx

Notes
-----

"""

#mnemonics underlaying certain conditions 15min
# INRSD_EXP_STAT != STARTED
mnemonic_cond_1 = [
"SE_ZINRSFPEA",
"SE_ZINRSFPEB"]

#mnemonics underlaying condition 15min
# INRSH_LAMP_SEL = NO_LAMP
mnemonic_cond_2 = [
"SE_ZINRSICEA",
"SE_ZINRSICEB"]

mnemonic_cond_3 = [
"SE_ZINRSMCEA",
"SE_ZINRSMCEB"]

#menmonics applicable when CAA is powered
#INRSH_CAA_PWRF_ST = ON
mnemonic_caa= [
"IGDP_NRSI_C_CAAL1_TEMP",
"IGDP_NRSI_C_CAAL2_TEMP",
"IGDP_NRSI_C_CAAL3_TEMP",
"IGDP_NRSI_C_CAAL4_TEMP"]

#only applicable when Filter table 10 is set
mnemonic_ft10 = [
"INRSH_OA_VREFOFF",
"INRSH_OA_VREF",

"INRSH_CAA_VREFOFF",
"INRSH_CAA_VREF",

"INRSH_FWA_ADCMGAIN",
"INRSH_FWA_ADCMOFFSET",
"INRSH_FWA_MOTOR_VREF",

"INRSH_GWA_ADCMGAIN",
"INRSH_GWA_ADCMOFFSET",
"INRSH_GWA_MOTOR_VREF",

"INRSH_RMA_ADCMGAIN",
"INRSH_RMA_ADCMOFFSET"]

#all mnemonics used for conditions
mnemonic_for_conditions = [
"INRSM_MOVE_STAT",
"INRSH_WHEEL_MOT_SVREF",
"INRSI_CAA_ON_FLAG",
"INRSH_LAMP_SEL",
"INRSD_EXP_STAT",

"INRSH_CAA_PWRF_ST",

"INRSI_FWA_MOVE_ST",
"INRSI_FWA_MECH_POS",
"INRSI_GWA_MOVE_ST",
"INRSI_GWA_MECH_POS",

"INRSI_C_FWA_POSITION",
"INRSI_C_GWA_X_POSITION",
"INRSI_C_GWA_Y_POSITION",

"ICTM_RT_FILTER" ]

#these mnemonic are used by the day routine
mnemSet_day = [
"INRSM_MSA_Q1_365VDD",
"INRSM_MSA_Q1_365VPP",
"INRSM_MSA_Q1_171VPP",
"IGDPM_MSA_Q1_365IDD",
"IGDPM_MSA_Q1_365IPP",
"IGDPM_MSA_Q1_171RTN",

"INRSM_MSA_Q2_365VDD",
"INRSM_MSA_Q2_365VPP",
"INRSM_MSA_Q2_171VPP",
"IGDPM_MSA_Q2_365IDD",
"IGDPM_MSA_Q2_365IPP",
"IGDPM_MSA_Q2_171RTN",

"INRSM_MSA_Q3_365VDD",
"INRSM_MSA_Q3_365VPP",
"INRSM_MSA_Q3_171VPP",
"IGDPM_MSA_Q3_365IDD",
"IGDPM_MSA_Q3_365IPP",
"IGDPM_MSA_Q3_171RTN",

"INRSM_MSA_Q4_365VDD",
"INRSM_MSA_Q4_365VPP",
"INRSM_MSA_Q4_171VPP",
"IGDPM_MSA_Q4_365IDD",
"IGDPM_MSA_Q4_365IPP",
"IGDPM_MSA_Q4_171RTN",

"IGDP_NRSD_ALG_FPA_TEMP",
"IGDP_NRSD_ALG_A1_TEMP",
"IGDP_NRSD_ALG_A2_TEMP",
"IGDP_NRSI_C_FWA_TEMP",
"IGDP_NRSI_C_GWA_TEMP",

"SI_GZCTS74A",
"SI_GZCTS74B",
"SI_GZCTS67A",
"SI_GZCTS67B"]

#these mnemonic are used by the 15min routine
mnemSet_15min = [
"IGDP_NRSD_ALG_TEMP",

"INRSD_ALG_ACC_P12C",
"INRSD_ALG_ACC_N12C",
"INRSD_ALG_ACC_3D3_1D5_C",
"INRSD_ALG_CHASSIS",

"IGDP_NRSD_ALG_A1_VDD_C",
"IGDP_NRSD_ALG_A1_VDDA",
"IGDP_NRSD_ALG_A1VDAP12C",
"IGDP_NRSD_ALG_A1VDAN12C",
"IGDP_NRSD_ALG_A1GND4VDA",
"IGDP_NRSD_ALG_A1GND5VRF",
"INRSD_ALG_A1_VDD3P3",
"INRSD_ALG_A1_VDD",
"INRSD_ALG_A1_REF",
"INRSD_A1_DSUB_V",
"INRSD_A1_VRESET_V",
"INRSD_A1_CELLDRN_V",
"INRSD_A1_DRAIN_V",
"INRSD_A1_VBIASGATE_V",
"INRSD_A1_VBIASPWR_V",
"INRSD_A1_VDDA_I",

"IGDP_NRSD_ALG_A2_VDD_C",
"IGDP_NRSD_ALG_A2_VDDA",
"IGDP_NRSD_ALG_A2VDAP12C",
"IGDP_NRSD_ALG_A2VDAN12C",
"IGDP_NRSD_ALG_A2GND4VDA",
"IGDP_NRSD_ALG_A2GND5VRF",
"INRSD_ALG_A2_VDD3P3",
"INRSD_ALG_A2_VDD",
"INRSD_ALG_A2_REF",
"INRSD_A2_DSUB_V",
"INRSD_A2_VRESET_V",
"INRSD_A2_CELLDRN_V",
"INRSD_A2_DRAIN_V",
"INRSD_A2_VBIASGATE_V",
"INRSD_A2_VBIASPWR_V",
"INRSD_A2_VDDA_I",

"INRSH_HK_TEMP1",
"INRSH_HK_TEMP2",

"INRSH_HK_P15V",
"INRSH_HK_N15V",
"INRSH_HK_VMOTOR",
"INRSH_HK_P5V",
"INRSH_HK_2P5V",
"INRSH_HK_ADCTGAIN",
"INRSH_HK_ADCTOFFSET",

"IGDP_NRSI_C_CAM_TEMP",
"IGDP_NRSI_C_COL_TEMP",
"IGDP_NRSI_C_COM1_TEMP",
"IGDP_NRSI_C_FOR_TEMP",
"IGDP_NRSI_C_IFU_TEMP",
"IGDP_NRSI_C_BP1_TEMP",
"IGDP_NRSI_C_BP2_TEMP",
"IGDP_NRSI_C_BP3_TEMP",
"IGDP_NRSI_C_BP4_TEMP",
"IGDP_NRSI_C_RMA_TEMP",

"SI_GZCTS75A",
"SI_GZCTS68A",
"SI_GZCTS81A",
"SI_GZCTS80A",
"SI_GZCTS70A",
"SI_GZCTS76A",
"SI_GZCTS79A",
"SI_GZCTS77A",
"SI_GZCTS78A",
"SI_GZCTS69A",

"INRSM_MCE_AIC_1R5_V",
"INRSM_MCE_AIC_3R3_V",
"INRSM_MCE_AIC_5_V",
"INRSM_MCE_AIC_P12_V",
"INRSM_MCE_AIC_N12_V",
"INRSM_MCE_AIC_3R3_I",
"INRSM_MCE_AIC_5_I",
"INRSM_MCE_AIC_P12_I",
"INRSM_MCE_AIC_N12_I",

"INRSM_MCE_MDAC_1R5_V",
"INRSM_MCE_MDAC_3R3_V",
"INRSM_MCE_MDAC_5_V",
"INRSM_MCE_MDAC_P12_V",
"INRSM_MCE_MDAC_N12_V",
"INRSM_MCE_MDAC_3R3_I",
"INRSM_MCE_MDAC_5_I",
"INRSM_MCE_MDAC_P12_I",
"INRSM_MCE_MDAC_N12_I",

"INRSM_MCE_PCA_TMP1",
"INRSM_MCE_PCA_TMP2",
"INRSM_MCE_AIC_TMP_FPGA",
"INRSM_MCE_AIC_TMP_ADC",
"INRSM_MCE_AIC_TMP_VREG",
"INRSM_MCE_MDAC_TMP_FPGA",
"INRSM_MCE_MDAC_TMP_OSC",
"INRSM_MCE_MDAC_TMP_BRD",
"INRSM_MCE_MDAC_TMP_PHA",
"INRSM_MCE_MDAC_TMP_PHB",

"INRSM_Q1_TMP_A",
"INRSM_Q2_TMP_A",
"INRSM_Q3_TMP_A",
"INRSM_Q4_TMP_A",
"INRSM_MECH_MTR_TMP_A",
"INRSM_LL_MTR_TMP_A",
"INRSM_MSA_TMP_A"]

#mnemonic set for setting up database
mnemonic_set_database = [
"GP_ZPSVOLT",
"SE_ZINRSFPEA",
"SE_ZINRSFPEB",

"IGDP_NRSD_ALG_TEMP",

"IGDP_NRSD_ALG_FPA_TEMP",
"IGDP_NRSD_ALG_A1_TEMP",
"IGDP_NRSD_ALG_A2_TEMP",
"SI_GZCTS74A",
"SI_GZCTS74B",
"SI_GZCTS67A",
"SI_GZCTS67B",

"INRSD_ALG_ACC_P12C",
"INRSD_ALG_ACC_N12C",
"INRSD_ALG_ACC_3D3_1D5_C",
"INRSD_ALG_CHASSIS",

"IGDP_NRSD_ALG_A1_VDD_C",
"IGDP_NRSD_ALG_A1_VDDA",
"IGDP_NRSD_ALG_A1VDAP12C",
"IGDP_NRSD_ALG_A1VDAN12C",
"IGDP_NRSD_ALG_A1GND4VDA",
"IGDP_NRSD_ALG_A1GND5VRF",
"INRSD_ALG_A1_VDD3P3",
"INRSD_ALG_A1_VDD",
"INRSD_ALG_A1_REF",
"INRSD_A1_DSUB_V",
"INRSD_A1_VRESET_V",
"INRSD_A1_CELLDRN_V",
"INRSD_A1_DRAIN_V",
"INRSD_A1_VBIASGATE_V",
"INRSD_A1_VBIASPWR_V",
"INRSD_A1_VDDA_I",

"IGDP_NRSD_ALG_A2_VDD_C",
"IGDP_NRSD_ALG_A2_VDDA",
"IGDP_NRSD_ALG_A2VDAP12C",
"IGDP_NRSD_ALG_A2VDAN12C",
"IGDP_NRSD_ALG_A2GND4VDA",
"IGDP_NRSD_ALG_A2GND5VRF",
"INRSD_ALG_A2_VDD3P3",
"INRSD_ALG_A2_VDD",
"INRSD_ALG_A2_REF",
"INRSD_A2_DSUB_V",
"INRSD_A2_VRESET_V",
"INRSD_A2_CELLDRN_V",
"INRSD_A2_DRAIN_V",
"INRSD_A2_VBIASGATE_V",
"INRSD_A2_VBIASPWR_V",
"INRSD_A2_VDDA_I",

"SE_ZINRSICEA",
"SE_ZINRSICEB",

"INRSH_HK_TEMP1",
"INRSH_HK_TEMP2",

"INRSH_HK_P15V",
"INRSH_HK_N15V",
"INRSH_HK_VMOTOR",
"INRSH_HK_P5V",
"INRSH_HK_2P5V",
"INRSH_HK_ADCTGAIN",
"INRSH_HK_ADCTOFFSET",

"INRSH_OA_VREFOFF",
"INRSH_OA_VREF",

"IGDP_NRSI_C_CAM_TEMP",
"IGDP_NRSI_C_COL_TEMP",
"IGDP_NRSI_C_COM1_TEMP",
"IGDP_NRSI_C_FOR_TEMP",
"IGDP_NRSI_C_IFU_TEMP",
"IGDP_NRSI_C_BP1_TEMP",
"IGDP_NRSI_C_BP2_TEMP",
"IGDP_NRSI_C_BP3_TEMP",
"IGDP_NRSI_C_BP4_TEMP",
"IGDP_NRSI_C_RMA_TEMP",

"INRSH_CAA_VREFOFF",
"INRSH_CAA_VREF",

"INRSH_LAMP_SEL",
"INRSI_C_CAA_CURRENT",
"INRSI_C_CAA_VOLTAGE",

"IGDP_NRSI_C_CAAL1_TEMP",
"IGDP_NRSI_C_CAAL2_TEMP",
"IGDP_NRSI_C_CAAL3_TEMP",
"IGDP_NRSI_C_CAAL4_TEMP",

"INRSH_FWA_ADCMGAIN",
"INRSH_FWA_ADCMOFFSET",
"INRSH_FWA_MOTOR_VREF",

"IGDP_NRSI_C_FWA_TEMP",

"INRSH_GWA_ADCMGAIN",
"INRSH_GWA_ADCMOFFSET",
"INRSH_GWA_MOTOR_VREF",

"IGDP_NRSI_C_GWA_TEMP",

"INRSH_RMA_ADCMGAIN",
"INRSH_RMA_ADCMOFFSET",

"SI_GZCTS75A",
"SI_GZCTS68A",
"SI_GZCTS81A",
"SI_GZCTS80A",
"SI_GZCTS70A",
"SI_GZCTS76A",
"SI_GZCTS79A",
"SI_GZCTS77A",
"SI_GZCTS78A",
"SI_GZCTS69A",
"SI_GZCTS75B",
"SI_GZCTS68B",
"SI_GZCTS81B",
"SI_GZCTS80B",
"SI_GZCTS70B",
"SI_GZCTS76B",
"SI_GZCTS79B",
"SI_GZCTS77B",
"SI_GZCTS78B",
"SI_GZCTS69B",

"SE_ZINRSMCEA",
"SE_ZINRSMCEB",

"INRSM_MCE_AIC_1R5_V",
"INRSM_MCE_AIC_3R3_V",
"INRSM_MCE_AIC_5_V",
"INRSM_MCE_AIC_P12_V",
"INRSM_MCE_AIC_N12_V",
"INRSM_MCE_AIC_3R3_I",
"INRSM_MCE_AIC_5_I",
"INRSM_MCE_AIC_P12_I",
"INRSM_MCE_AIC_N12_I",

"INRSM_MCE_MDAC_1R5_V",
"INRSM_MCE_MDAC_3R3_V",
"INRSM_MCE_MDAC_5_V",
"INRSM_MCE_MDAC_P12_V",
"INRSM_MCE_MDAC_N12_V",
"INRSM_MCE_MDAC_3R3_I",
"INRSM_MCE_MDAC_5_I",
"INRSM_MCE_MDAC_P12_I",
"INRSM_MCE_MDAC_N12_I",

"INRSM_MCE_PCA_TMP1",
"INRSM_MCE_PCA_TMP2",
"INRSM_MCE_AIC_TMP_FPGA",
"INRSM_MCE_AIC_TMP_ADC",
"INRSM_MCE_AIC_TMP_VREG",
"INRSM_MCE_MDAC_TMP_FPGA",
"INRSM_MCE_MDAC_TMP_OSC",
"INRSM_MCE_MDAC_TMP_BRD",
"INRSM_MCE_MDAC_TMP_PHA",
"INRSM_MCE_MDAC_TMP_PHB",

"INRSM_Q1_TMP_A",
"INRSM_Q2_TMP_A",
"INRSM_Q3_TMP_A",
"INRSM_Q4_TMP_A",
"INRSM_MECH_MTR_TMP_A",
"INRSM_LL_MTR_TMP_A",
"INRSM_MSA_TMP_A",

"INRSM_Q1_TMP_B",
"INRSM_Q2_TMP_B",
"INRSM_Q3_TMP_B",
"INRSM_Q4_TMP_B",
"INRSM_MECH_MTR_TMP_B",
"INRSM_LL_MTR_TMP_B",
"INRSM_MSA_TMP_B",

"INRSM_MSA_Q1_365VDD",
"INRSM_MSA_Q1_365VPP",
"INRSM_MSA_Q1_171VPP",
"IGDPM_MSA_Q1_365IDD",
"IGDPM_MSA_Q1_365IPP",
"IGDPM_MSA_Q1_171RTN",

"INRSM_MSA_Q2_365VDD",
"INRSM_MSA_Q2_365VPP",
"INRSM_MSA_Q2_171VPP",
"IGDPM_MSA_Q2_365IDD",
"IGDPM_MSA_Q2_365IPP",
"IGDPM_MSA_Q2_171RTN",

"INRSM_MSA_Q3_365VDD",
"INRSM_MSA_Q3_365VPP",
"INRSM_MSA_Q3_171VPP",
"IGDPM_MSA_Q3_365IDD",
"IGDPM_MSA_Q3_365IPP",
"IGDPM_MSA_Q3_171RTN",

"INRSM_MSA_Q4_365VDD",
"INRSM_MSA_Q4_365VPP",
"INRSM_MSA_Q4_171VPP",
"IGDPM_MSA_Q4_365IDD",
"IGDPM_MSA_Q4_365IPP",
"IGDPM_MSA_Q4_171RTN",

"LAMP_FLAT1_CURR",
"LAMP_FLAT2_CURR",
"LAMP_FLAT3_CURR",
"LAMP_FLAT4_CURR",
"LAMP_FLAT5_CURR",
"LAMP_LINE1_CURR",
"LAMP_LINE2_CURR",
"LAMP_LINE3_CURR",
"LAMP_LINE4_CURR",
"LAMP_REF_CURR",
"LAMP_TEST_CURR",

"LAMP_FLAT1_VOLT",
"LAMP_FLAT2_VOLT",
"LAMP_FLAT3_VOLT",
"LAMP_FLAT4_VOLT",
"LAMP_FLAT5_VOLT",
"LAMP_LINE1_VOLT",
"LAMP_LINE2_VOLT",
"LAMP_LINE3_VOLT",
"LAMP_LINE4_VOLT",
"LAMP_REF_VOLT",
"LAMP_TEST_VOLT"]

mnemonic_wheelpositions = [
"INRSI_C_FWA_POSITION_F110W",
"INRSI_C_FWA_POSITION_F100LP",
"INRSI_C_FWA_POSITION_F140X",
"INRSI_C_FWA_POSITION_OPAQUE",
"INRSI_C_FWA_POSITION_F290LP",
"INRSI_C_FWA_POSITION_F170LP",
"INRSI_C_FWA_POSITION_CLEAR",
"INRSI_C_FWA_POSITION_F070LP",

"INRSI_C_GWA_X_POSITION_PRISM",
"INRSI_C_GWA_Y_POSITION_PRISM",

"INRSI_C_GWA_X_POSITION_MIRROR",
"INRSI_C_GWA_Y_POSITION_MIRROR",

"INRSI_C_GWA_X_POSITION_G140H",
"INRSI_C_GWA_Y_POSITION_G140H",

"INRSI_C_GWA_X_POSITION_G235H",
"INRSI_C_GWA_Y_POSITION_G235H",

"INRSI_C_GWA_X_POSITION_G395H",
"INRSI_C_GWA_Y_POSITION_G395H",

"INRSI_C_GWA_X_POSITION_G140M",
"INRSI_C_GWA_Y_POSITION_G140M",

"INRSI_C_GWA_X_POSITION_G235M",
"INRSI_C_GWA_Y_POSITION_G235M",

"INRSI_C_GWA_X_POSITION_G395M",
"INRSI_C_GWA_Y_POSITION_G395M" ]

fw_nominals = {
'F110W': -123.99,
'F100LP' : -10.32,
'CLEAR' : -56.44,
'F070LP' : 43.45,
'F140X' : -78.37,
'OPAQUE' : 21.58,
'F290LP' : -95.78,
'F170LP' : 8.95}

gwx_nominals = {
'PRISM' : 169.01,
'MIRROR' : 171.11,
'G140H' : 180.25,
'G235H' : 176.66,
'G395H' : 159.96,
'G140M' : 164.31,
'G235M' : 159.24,
'G395M' : 141.69}

gwy_nominals = {
'PRISM' : 17.08,
'MIRROR' : 98.72,
'G140H' : 67.47,
'G235H' : 70.00,
'G395H' : 73.29,
'G140M' : 63.18,
'G235M' : 69.81,
'G395M' : 89.57}

#use this list for query
mnemonic_set_query = mnemonic_set_database + mnemonic_for_conditions



#! /usr/bin/env python
"""Prepares plots for Temperature tab

    Module prepares plots for mnemonics below. Combines plots in a grid and
    returns tab object.

    Plot 1 - MCE Board 1 (AIC) Voltages
    INRSM_MCE_AIC_1R5_V
    INRSM_MCE_AIC_3R3_V
    INRSM_MCE_AIC_5_V
    INRSM_MCE_AIC_P12_V
    INRSM_MCE_AIC_N12_V

    Plot 2 - MCE Board 1 (AIC) Currents
    INRSM_MCE_AIC_3R3_I
    INRSM_MCE_AIC_5_I
    INRSM_MCE_AIC_P12_I
    INRSM_MCE_AIC_N12_I

    Plot 3 - MCE Board 2 (MDAC) Voltages
    INRSM_MCE_MDAC_1R5_V
    INRSM_MCE_MDAC_3R3_V
    INRSM_MCE_MDAC_5_V
    INRSM_MCE_MDAC_P12_V
    INRSM_MCE_MDAC_N12_V

    Plot 4 - MCE Board 2 (MDAC) Currents
    INRSM_MCE_MDAC_3R3_I
    INRSM_MCE_MDAC_5_I
    INRSM_MCE_MDAC_P12_I
    INRSM_MCE_MDAC_N12_I

    Plot (5-8) - QUAD (1-4)
    INRSM_MSA_Q(1-4)_365VDD
    INRSM_MSA_Q(1-4)_365VPP
    INRSM_MSA_Q(1-4)_171VPP
    IGDPM_MSA_Q(1-4)_365IDD
    IGDPM_MSA_Q(1-4)_365IPP
    IGDPM_MSA_Q(1-4)_171RTN

Authors
-------
    - Daniel Kühbacher

Use
---
    The functions within this module are intended to be imported and
    used by ``nirspec_dashboard.py``, e.g.:

    ::
        from .plots.msa_mce_tab import msa_mce_plots
        tab = msa_mce_plots(conn, start, end)

Dependencies
------------
    User must provide database "nirspec_database.db"

"""
import jwql.instrument_monitors.nirspec_monitors.data_trending.utils.sql_interface as sql
import jwql.instrument_monitors.nirspec_monitors.data_trending.plots.plot_functions as pf
from bokeh.models import LinearAxis, Range1d
from bokeh.plotting import figure
from bokeh.models.widgets import Panel, Tabs, Div
from bokeh.models import ColumnDataSource, HoverTool, Title
from bokeh.layouts import WidgetBox, gridplot, Column

import pandas as pd
import numpy as np

from astropy.time import Time


def aic_voltage(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 560,
                plot_height = 700,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Voltage (V)')

    p.grid.visible = True
    p.title.text = "MCE Board 1 (AIC)"
    p.add_layout(Title(text="Voltages", text_font_style="italic", text_font_size="12pt"), 'above')
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "1R5_V", "INRSM_MCE_AIC_1R5_V", start, end, conn, color = "red")
    b = pf.add_to_plot(p, "3R3_V", "INRSM_MCE_AIC_3R3_V", start, end, conn, color = "orange")
    c = pf.add_to_plot(p, "5_V", "INRSM_MCE_AIC_5_V", start, end, conn, color = "brown")
    d = pf.add_to_plot(p, "P12_V", "INRSM_MCE_AIC_P12_V", start, end, conn, color = "burlywood")
    e = pf.add_to_plot(p, "N12_V", "INRSM_MCE_AIC_N12_V", start, end, conn, color = "darkmagenta")

    pf.add_hover_tool(p,[a,b,c,d,e])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"

    return p

def aic_current(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 560,
                plot_height = 700,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Current (A)')

    p.grid.visible = True
    p.title.text = "MCE Board 1 (AIC)"
    p.add_layout(Title(text="Currents", text_font_style="italic", text_font_size="12pt"), 'above')
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "3R3_I", "INRSM_MCE_AIC_3R3_I", start, end, conn, color = "blue")
    b = pf.add_to_plot(p, "5_I", "INRSM_MCE_AIC_5_I", start, end, conn, color = "red")
    c = pf.add_to_plot(p, "P12_I", "INRSM_MCE_AIC_P12_I", start, end, conn, color = "green")
    d = pf.add_to_plot(p, "N12_I", "INRSM_MCE_AIC_N12_I", start, end, conn, color = "orange")

    pf.add_hover_tool(p,[a,b,c,d])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"

    return p

def mdac_voltage(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 560,
                plot_height = 700,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Voltage (V)')

    p.grid.visible = True
    p.title.text = "MCE Board 2 (MDAC)"
    p.add_layout(Title(text="Voltages", text_font_style="italic", text_font_size="12pt"), 'above')
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "1R5_V", "INRSM_MCE_MDAC_1R5_V", start, end, conn, color = "red")
    b = pf.add_to_plot(p, "3R3_V", "INRSM_MCE_MDAC_3R3_V", start, end, conn, color = "orange")
    c = pf.add_to_plot(p, "5_V", "INRSM_MCE_MDAC_5_V", start, end, conn, color = "brown")
    d = pf.add_to_plot(p, "P12_V", "INRSM_MCE_MDAC_P12_V", start, end, conn, color = "burlywood")
    e = pf.add_to_plot(p, "N12_V", "INRSM_MCE_MDAC_N12_V", start, end, conn, color = "darkmagenta")

    pf.add_hover_tool(p,[a,b,c,d,e])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"

    return p

def mdac_current(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 560,
                plot_height = 700,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Voltage (V)')

    p.grid.visible = True
    p.title.text = "MCE Board 2 (MDAC)"
    p.add_layout(Title(text="Currents", text_font_style="italic", text_font_size="12pt"), 'above')
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "3R3_I", "INRSM_MCE_MDAC_3R3_I", start, end, conn, color = "blue")
    b = pf.add_to_plot(p, "5_I", "INRSM_MCE_MDAC_5_I", start, end, conn, color = "red")
    c = pf.add_to_plot(p, "P12_I", "INRSM_MCE_MDAC_P12_I", start, end, conn, color = "green")
    d = pf.add_to_plot(p, "N12_I", "INRSM_MCE_MDAC_N12_I", start, end, conn, color = "orange")

    pf.add_hover_tool(p,[a,b,c,d])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"

    return p

def quad1_volt(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 560,
                plot_height = 500,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Voltage (V)')

    p.grid.visible = True
    p.title.text = "Quad 1"
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "365VDD", "INRSM_MSA_Q1_365VDD", start, end, conn, color = "red")
    b = pf.add_to_plot(p, "365VPP", "INRSM_MSA_Q1_365VPP", start, end, conn, color = "orange")
    c = pf.add_to_plot(p, "171VPP", "INRSM_MSA_Q1_171VPP", start, end, conn, color = "brown")
    d = pf.add_to_plot(p, "365IDD", "IGDPM_MSA_Q1_365IDD", start, end, conn, color = "burlywood")
    e = pf.add_to_plot(p, "365IPP", "IGDPM_MSA_Q1_365IPP", start, end, conn, color = "darkmagenta")
    f = pf.add_to_plot(p, "171RTN", "IGDPM_MSA_Q1_171RTN", start, end, conn, color = "blue")

    pf.add_hover_tool(p,[a,b,c,d,e,f])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"

    return p

def quad2_volt(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 560,
                plot_height = 500,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Voltage (V)')

    p.grid.visible = True
    p.title.text = "Quad 2"
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "365VDD", "INRSM_MSA_Q2_365VDD", start, end, conn, color = "red")
    b = pf.add_to_plot(p, "365VPP", "INRSM_MSA_Q2_365VPP", start, end, conn, color = "orange")
    c = pf.add_to_plot(p, "171VPP", "INRSM_MSA_Q2_171VPP", start, end, conn, color = "brown")
    d = pf.add_to_plot(p, "365IDD", "IGDPM_MSA_Q2_365IDD", start, end, conn, color = "burlywood")
    e = pf.add_to_plot(p, "365IPP", "IGDPM_MSA_Q2_365IPP", start, end, conn, color = "darkmagenta")
    f = pf.add_to_plot(p, "171RTN", "IGDPM_MSA_Q2_171RTN", start, end, conn, color = "blue")

    pf.add_hover_tool(p,[a,b,c,d,e,f])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"

    return p

def quad3_volt(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 560,
                plot_height = 500,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Voltage (V)')

    p.grid.visible = True
    p.title.text = "Quad 3"
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "365VDD", "INRSM_MSA_Q3_365VDD", start, end, conn, color = "red")
    b = pf.add_to_plot(p, "365VPP", "INRSM_MSA_Q3_365VPP", start, end, conn, color = "orange")
    c = pf.add_to_plot(p, "171VPP", "INRSM_MSA_Q3_171VPP", start, end, conn, color = "brown")
    d = pf.add_to_plot(p, "365IDD", "IGDPM_MSA_Q3_365IDD", start, end, conn, color = "burlywood")
    e = pf.add_to_plot(p, "365IPP", "IGDPM_MSA_Q3_365IPP", start, end, conn, color = "darkmagenta")
    f = pf.add_to_plot(p, "171RTN", "IGDPM_MSA_Q3_171RTN", start, end, conn, color = "blue")

    pf.add_hover_tool(p,[a,b,c,d,e,f])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"

    return p

def quad4_volt(conn, start, end):
    '''Create specific plot and return plot object
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : Plot object
        Bokeh plot
    '''

    # create a new plot with a title and axis labels
    p = figure( tools = "pan,wheel_zoom,box_zoom,reset,save",
                toolbar_location = "above",
                plot_width = 560,
                plot_height = 500,
                x_axis_type = 'datetime',
                output_backend = "webgl",
                x_axis_label = 'Date', y_axis_label='Voltage (V)')

    p.grid.visible = True
    p.title.text = "Quad 4"
    pf.add_basic_layout(p)

    a = pf.add_to_plot(p, "365VDD", "INRSM_MSA_Q4_365VDD", start, end, conn, color = "red")
    b = pf.add_to_plot(p, "365VPP", "INRSM_MSA_Q4_365VPP", start, end, conn, color = "orange")
    c = pf.add_to_plot(p, "171VPP", "INRSM_MSA_Q4_171VPP", start, end, conn, color = "brown")
    d = pf.add_to_plot(p, "365IDD", "IGDPM_MSA_Q4_365IDD", start, end, conn, color = "burlywood")
    e = pf.add_to_plot(p, "365IPP", "IGDPM_MSA_Q4_365IPP", start, end, conn, color = "darkmagenta")
    f = pf.add_to_plot(p, "171RTN", "IGDPM_MSA_Q4_171RTN", start, end, conn, color = "blue")

    pf.add_hover_tool(p,[a,b,c,d,e,f])

    p.legend.location = "bottom_right"
    p.legend.click_policy = "hide"

    return p

def msa_mce_plots(conn, start, end):
    '''Combines plots to a tab
    Parameters
    ----------
    conn : DBobject
        Connection object that represents database
    start : time
        Startlimit for x-axis and query (typ. datetime.now()- 4Months)
    end : time
        Endlimit for x-axis and query (typ. datetime.now())
    Return
    ------
    p : tab object
        used by dashboard.py to set up dashboard
    '''
    descr = Div(text=
    """
    <style>
    table, th, td {
      border: 1px solid black;
      background-color: #efefef;
      border-collapse: collapse;
      padding: 5px
    }
    table {
      border-spacing: 15px;
    }
    </style>

    <body>
    <table style="width:100%">
      <tr>
        <th><h6>Plotname</h6></th>
        <th><h6>Mnemonic</h6></th>
        <th><h6>Description</h6></th>
      </tr>
      <tr>
        <td>MCE Board 1 (AIC) Voltages</td>
        <td>INRSM_MCE_AIC_1R5_V<br>
            INRSM_MCE_AIC_3R3_V<br>
            INRSM_MCE_AIC_5_V<br>
            INRSM_MCE_AIC_P12_V<br>
            INRSM_MCE_AIC_N12_V<br>
        </td>
        <td>MCE AIC +1.5V Voltage<br>
            MCE AIC +3.3V Voltage<br>
            MCE AIC +5V Voltage<br>
            MCE AIC +12V Voltage<br>
            MCE AIC -12V Voltage<br>
        </td>
      </tr>

      <tr>
        <td>MCE Board 1 (AIC) Currents</td>
        <td>INRSM_MCE_AIC_3R3_I<br>
            INRSM_MCE_AIC_5_I<br>
            INRSM_MCE_AIC_P12_I<br>
            INRSM_MCE_AIC_N12_I<br>
        </td>
        <td>MCE AIC Board +3.3V Current<br>
            MCE AIC Board +5V Current<br>
            MCE AIC Board +12V Current<br>
            MCE AIC Board -12V Current<br>
        </td>
      </tr>

      <tr>
        <td>MCE Board 2 (MDAC) Voltages</td>
        <td>INRSM_MCE_MDAC_1R5_V<br>
            INRSM_MCE_MDAC_3R3_V<br>
            INRSM_MCE_MDAC_5_V<br>
            INRSM_MCE_MDAC_P12_V<br>
            INRSM_MCE_MDAC_N12_V<br>
        </td>
        <td>MCE MDAC +1.5V Voltage<br>
            MCE MDAC +3.3V Voltage<br>
            MCE MDAC +5V Voltage<br>
            MCE MDAC +12V Voltage<br>
            MCE MDAC -12V Voltage<br>
        </td>
      </tr>

      <tr>
        <td>MCE Board 2 (MDAC) Currents</td>
        <td>INRSM_MCE_MDAC_3R3_I<br>
            INRSM_MCE_MDAC_5_I<br>
            INRSM_MCE_MDAC_P12_I<br>
            INRSM_MCE_MDAC_N12_I<br>
        </td>
        <td>MCE MDAC Board +3.3V Current<br>
            MCE MDAC Board +5V Current<br>
            MCE MDAC Board +12V Current<br>
            MCE MDAC Board -12V Current<br>
        </td>
      </tr>

      <tr>
        <td>QUAD (1-4)</td>
        <td>INRSM_MSA_Q(1-4)_365VDD<br>
            INRSM_MSA_Q(1-4)_365VPP<br>
            INRSM_MSA_Q(1-4)_171VPP<br>
            IGDPM_MSA_Q(1-4)_365IDD<br>
            IGDPM_MSA_Q(1-4)_365IPP<br>
            IGDPM_MSA_Q(1-4)_171RTN<br>
        </td>
        <td>MSA Quad (1-4) Vdd 365 Voltage<br>
            MSA Quad (1-4) Vpp 365 Voltage<br>
            MSA Quad (1-4) Vpp 171 Voltage<br>
            MSA Quad (1-4) Vdd 365 Current<br>
            MSA Quad (1-4) Vpp 365 Current<br>
            MSA Quad (1-4) Return 171 Current<br>
        </td>
      </tr>
    </table>
    </body>
    """, width=1100)

    plot1 = aic_voltage(conn, start, end)
    plot2 = aic_current(conn, start, end)
    plot3 = mdac_voltage(conn, start, end)
    plot4 = mdac_current(conn, start, end)
    plot5 = quad1_volt(conn, start, end)
    plot6 = quad2_volt(conn, start, end)
    plot7 = quad3_volt(conn, start, end)
    plot8 = quad4_volt(conn, start, end)

    grid = gridplot([[plot1, plot2],
                    [plot3, plot4],
                    [plot5, plot6],
                    [plot7, plot8]],merge_tools=False)
    layout = Column(descr, grid)

    tab = Panel(child = layout, title = "MSA/MCE")

    return tab



import numpy as np
from setuptools import setup
from setuptools import find_packages

VERSION = '0.19.0'

AUTHORS = 'Matthew Bourque, Lauren Chambers, Misty Cracraft, Joe Filippazzo, Bryan Hilbert, '
AUTHORS += 'Graham Kanarek, Catherine Martlin, Johannes Sahlmann'

DESCRIPTION = 'The James Webb Space Telescope Quicklook Project'

REQUIRES = [
    'astropy',
    'astroquery>=0.3.9',
    'authlib',
    'bokeh>=1.0',
    'django>=2.0',
    'jinja2',
    'jwedb',
    'jwst',
    'matplotlib',
    'numpy',
    'numpydoc',
    'pandas',
    'psycopg2',
    'pysiaf',
    'pytest',
    'sphinx',
    'sqlalchemy',
    'stsci_rtd_theme'
]

setup(
    name='jwql',
    version=VERSION,
    description=DESCRIPTION,
    url='https://github.com/spacetelescope/jwql.git',
    author=AUTHORS,
    author_email='jwql@stsci.edu',
    license='BSD',
    keywords=['astronomy', 'python'],
    classifiers=['Programming Language :: Python'],
    packages=find_packages(),
    install_requires=REQUIRES,
    include_package_data=True,
    include_dirs=[np.get_include()],
)
